import { CommonModule } from '@angular/common';
import { Injectable, Injector, NgModule } from '@angular/core';
import { validateSync } from 'class-validator';
import { HttpHeaders, HttpClient } from '@angular/common/http';
import * as parseImported from 'url-parse';
import { plainToClassFromExist, classToClass, classToPlainFromExist } from 'class-transformer';
import { List } from 'immutable';
import { Observable, BehaviorSubject, Subject, of, throwError } from 'rxjs';
import { delay, first, map, takeUntil } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {number} */
const ProviderActionEnum = {
    Action: 0,
    Create: 1,
    Append: 2,
    Update: 3,
    Patch: 4,
    Delete: 5,
    Load: 6,
    LoadAll: 7,
};
ProviderActionEnum[ProviderActionEnum.Action] = 'Action';
ProviderActionEnum[ProviderActionEnum.Create] = 'Create';
ProviderActionEnum[ProviderActionEnum.Append] = 'Append';
ProviderActionEnum[ProviderActionEnum.Update] = 'Update';
ProviderActionEnum[ProviderActionEnum.Patch] = 'Patch';
ProviderActionEnum[ProviderActionEnum.Delete] = 'Delete';
ProviderActionEnum[ProviderActionEnum.Load] = 'Load';
ProviderActionEnum[ProviderActionEnum.LoadAll] = 'LoadAll';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PaginationMeta {
    /**
     * @param {?=} data
     */
    constructor(data) {
        plainToClassFromExist(this, data);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class RestProviderActionHandlers {
    /**
     * @param {?} requestUrl
     * @param {?} data
     * @param {?} requestOptions
     * @param {?} optionsList
     * @param {?} action
     * @param {?=} useDefault
     * @return {?}
     */
    getRequest(requestUrl, data, requestOptions, optionsList, action, useDefault) {
        /** @type {?} */
        let request;
        if (useDefault !== true) {
            optionsList.forEach(eachOptions => {
                if (eachOptions !== undefined &&
                    eachOptions.actionOptions !== undefined &&
                    eachOptions.actionOptions.request !== undefined) {
                    request = eachOptions.actionOptions.request;
                    return request;
                }
            });
        }
        if (request !== undefined) {
            return request(requestUrl, data, requestOptions);
        }
        return undefined;
    }
    /**
     * @param {?} data
     * @param {?} optionsList
     * @param {?} action
     * @param {?=} useDefault
     * @return {?}
     */
    getResponseData(data, optionsList, action, useDefault) {
        /** @type {?} */
        let responseData;
        if (useDefault !== true) {
            optionsList.forEach(eachOptions => {
                if (eachOptions !== undefined &&
                    eachOptions.actionOptions !== undefined &&
                    eachOptions.actionOptions.responseData !== undefined) {
                    responseData = eachOptions.actionOptions.responseData(data, action);
                    return responseData;
                }
            });
        }
        if (responseData !== undefined) {
            return responseData;
        }
        return data.body;
    }
    /**
     * @param {?} data
     * @param {?} optionsList
     * @param {?} action
     * @param {?=} useDefault
     * @return {?}
     */
    getResponsePaginationMeta(data, optionsList, action, useDefault) {
        /** @type {?} */
        let paginationMeta;
        if (useDefault !== true) {
            optionsList.forEach(eachOptions => {
                if (eachOptions !== undefined &&
                    eachOptions.actionOptions !== undefined &&
                    eachOptions.actionOptions.responsePaginationMeta !== undefined) {
                    paginationMeta = eachOptions.actionOptions.responsePaginationMeta(data, action);
                    return paginationMeta;
                }
            });
        }
        if (paginationMeta !== undefined) {
            return paginationMeta;
        }
        paginationMeta = new PaginationMeta();
        /** @type {?} */
        let xTotalCount = NaN;
        /** @type {?} */
        const keys = data.headers ? (data.headers.keys ? data.headers.keys() : data.headers) : [];
        keys.forEach(key => {
            if (key.toLowerCase() === 'x-total-count') {
                xTotalCount = +data.headers.get(key);
            }
        });
        paginationMeta.totalResults = isNaN(xTotalCount) ? 10000 : xTotalCount;
        return paginationMeta;
    }
    /**
     * @param {?} currentUrl
     * @param {?} filter
     * @param {?} optionsList
     * @param {?} action
     * @param {?=} useDefault
     * @return {?}
     */
    getRequestQuery(currentUrl, filter, optionsList, action, useDefault) {
        /** @type {?} */
        let query;
        if (useDefault !== true) {
            optionsList.forEach(eachOptions => {
                if (eachOptions !== undefined &&
                    eachOptions.actionOptions !== undefined &&
                    eachOptions.actionOptions.requestQuery !== undefined) {
                    query = eachOptions.actionOptions.requestQuery(currentUrl, filter, action);
                    return query;
                }
            });
        }
        if (query !== undefined) {
            return query;
        }
        for (const key in filter) {
            if (filter.hasOwnProperty(key)) {
                /** @type {?} */
                const value = filter[key];
                if (!currentUrl.includes(`?${key}=`) && !currentUrl.includes(`&${key}=`)) {
                    currentUrl = currentUrl + ((currentUrl.indexOf('?') === -1 ? '?' : '&') + `${key}=${filter[key]}`);
                }
            }
        }
        return currentUrl;
    }
    /**
     * @param {?} key
     * @param {?} data
     * @param {?} optionsList
     * @param {?} action
     * @param {?=} useDefault
     * @return {?}
     */
    getRequestUrl(key, data, optionsList, action, useDefault) {
        /** @type {?} */
        let requestUrl;
        optionsList.forEach(eachOptions => {
            if (eachOptions !== undefined &&
                eachOptions.actionOptions !== undefined &&
                eachOptions.actionOptions.requestUrl !== undefined) {
                requestUrl = eachOptions.actionOptions.requestUrl(key, data, action);
                return requestUrl;
            }
        });
        if (requestUrl !== undefined) {
            return requestUrl;
        }
        /** @type {?} */
        let apiUrl = '';
        /** @type {?} */
        let pluralName = '';
        if (useDefault !== true) {
            optionsList.forEach(eachOptions => {
                if (eachOptions !== undefined && eachOptions.apiUrl !== undefined) {
                    apiUrl = eachOptions.apiUrl;
                }
                if (eachOptions !== undefined && eachOptions.pluralName !== undefined) {
                    pluralName = eachOptions.pluralName;
                }
            });
        }
        if (action === ProviderActionEnum.Action) {
            return apiUrl + '/' + pluralName + '/' + key;
        }
        if (action === ProviderActionEnum.Create) {
            return apiUrl + '/' + pluralName;
        }
        if (action === ProviderActionEnum.Update) {
            return apiUrl + '/' + pluralName + '/' + key;
        }
        if (action === ProviderActionEnum.Patch) {
            return apiUrl + '/' + pluralName + '/' + key;
        }
        if (action === ProviderActionEnum.Load) {
            return apiUrl + '/' + pluralName + '/' + key;
        }
        if (action === ProviderActionEnum.Delete) {
            return apiUrl + '/' + pluralName + '/' + key;
        }
        if (action === ProviderActionEnum.LoadAll) {
            return apiUrl + '/' + pluralName;
        }
    }
    /**
     * @param {?} key
     * @param {?} data
     * @param {?} optionsList
     * @param {?} action
     * @param {?=} useDefault
     * @return {?}
     */
    getRequestOptions(key, data, optionsList, action, useDefault) {
        /** @type {?} */
        const requestOptions = {
            observe: 'response'
        };
        if (useDefault !== true) {
            optionsList.reverse().forEach(eachOptions => {
                if (eachOptions !== undefined &&
                    eachOptions.actionOptions !== undefined &&
                    eachOptions.actionOptions.requestOptions !== undefined) {
                    /** @type {?} */
                    const actionOptionsRequestOptions = eachOptions.actionOptions.requestOptions(key, data, action);
                    if (actionOptionsRequestOptions.headers !== undefined) {
                        requestOptions.headers = actionOptionsRequestOptions.headers;
                    }
                    if (actionOptionsRequestOptions.observe !== undefined) {
                        requestOptions.observe = actionOptionsRequestOptions.observe;
                    }
                    if (actionOptionsRequestOptions.params !== undefined) {
                        requestOptions.params = actionOptionsRequestOptions.params;
                    }
                    if (actionOptionsRequestOptions.reportProgress !== undefined) {
                        requestOptions.reportProgress = actionOptionsRequestOptions.reportProgress;
                    }
                    if (actionOptionsRequestOptions.responseType !== undefined) {
                        requestOptions.responseType = actionOptionsRequestOptions.responseType;
                    }
                    if (actionOptionsRequestOptions.withCredentials !== undefined) {
                        requestOptions.withCredentials = actionOptionsRequestOptions.withCredentials;
                    }
                }
            });
        }
        return (/** @type {?} */ (requestOptions));
    }
    /**
     * @param {?} optionsList
     * @param {?} action
     * @param {?=} useDefault
     * @return {?}
     */
    getRequestCreateType(optionsList, action, useDefault) {
        /** @type {?} */
        let requestCreateType;
        if (useDefault !== true) {
            optionsList.forEach(eachOptions => {
                if (eachOptions !== undefined &&
                    eachOptions.actionOptions !== undefined &&
                    eachOptions.actionOptions.requestCreateType !== undefined) {
                    requestCreateType = eachOptions.actionOptions.requestCreateType(action);
                    return requestCreateType;
                }
            });
        }
        if (requestCreateType !== undefined) {
            return requestCreateType;
        }
        return 'create';
    }
}
RestProviderActionHandlers.decorators = [
    { type: Injectable }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class MockProviderActionHandlers extends RestProviderActionHandlers {
}
MockProviderActionHandlers.decorators = [
    { type: Injectable }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class Model {
}
Model.strings = {};
Model.fields = [];
Model.nested = {};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ProviderError extends Error {
    /**
     * @param {?} error
     */
    constructor(error) {
        super(error instanceof Error ? error.message : ((/** @type {?} */ (error))));
        this.error = error;
        // Set the prototype explicitly.
        Object.setPrototypeOf(this, ProviderError.prototype);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ValidatorError extends Error {
    /**
     * @param {?} errors
     */
    constructor(errors) {
        super(Array.isArray(errors) ? JSON.stringify(errors) : ((/** @type {?} */ (errors))));
        this.errors = errors;
        // Set the prototype explicitly.
        Object.setPrototypeOf(this, ValidatorError.prototype);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const parse = parseImported;
class FakeHttpClient {
    /**
     * @param {?=} items
     * @param {?=} options
     */
    constructor(items, options) {
        this._idField = 'id';
        this._apiUrlPrefix = '/api/';
        this._pageQueryParam = 'page';
        this._limitQueryParam = 'limit';
        this._searchTextQueryParam = 'search';
        if (options) {
            if (options.idField !== undefined) {
                this._idField = options.idField;
            }
            if (options.apiUrlPrefix !== undefined) {
                this._apiUrlPrefix = options.apiUrlPrefix;
            }
            if (options.pageQueryParam !== undefined) {
                this._pageQueryParam = options.pageQueryParam;
            }
            if (options.limitQueryParam !== undefined) {
                this._limitQueryParam = options.limitQueryParam;
            }
            if (options.searchTextQueryParam !== undefined) {
                this._searchTextQueryParam = options.searchTextQueryParam;
            }
        }
        if (items !== undefined) {
            this.setItems(items);
        }
        else {
            this.mockedItems = List([]);
        }
    }
    /**
     * @param {?} items
     * @return {?}
     */
    setItems(items) {
        this.mockedItems = List(items);
    }
    /**
     * @return {?}
     */
    genId() {
        /** @type {?} */
        let currentMax = 0;
        this.mockedItems.map((mapItem) => Math.abs(mapItem[this._idField]) > currentMax ? (currentMax = Math.abs(mapItem[this._idField])) : null);
        return (currentMax + 1) * -1;
    }
    /**
     * @param {?} url
     * @param {?=} data
     * @return {?}
     */
    parseUrl(url, data) {
        /** @type {?} */
        const uri = parse(url, {}, true);
        /** @type {?} */
        const arrWithoutApi = uri.pathname.split(this._apiUrlPrefix);
        /** @type {?} */
        const arr = arrWithoutApi[arrWithoutApi.length - 1].split('/');
        /** @type {?} */
        const entityName = arr.length === 2 ? arr[arr.length - 2] : arr[arr.length - 1];
        /** @type {?} */
        const entityKey = arr.length === 2 ? arr[arr.length - 1] : undefined;
        /** @type {?} */
        const page = uri.query[this._pageQueryParam];
        /** @type {?} */
        const limit = uri.query[this._limitQueryParam];
        /** @type {?} */
        const searchText = uri.query[this._searchTextQueryParam];
        return {
            name: entityName,
            key: entityKey,
            query: uri.query,
            uri: uri,
            page: page,
            limit: limit,
            searchText: searchText
        };
    }
    /**
     * @param {?} data
     * @param {?} word
     * @return {?}
     */
    checkWordInData(data, word) {
        /** @type {?} */
        const result = Object.keys(data).filter(key => {
            if (data[key] instanceof Object) {
                return this.checkWordInData(data[key], word);
            }
            if (!isNaN(+data[key])) {
                return +data[key] === +word;
            }
            return data[key] ? ((/** @type {?} */ (data[key]))).toLowerCase().indexOf(word.toLowerCase()) !== -1 : false;
        });
        return result.length > 0;
    }
    /**
     * @template T
     * @param {?} url
     * @param {?=} options
     * @return {?}
     */
    get(url, options) {
        return new Observable(observer => {
            /** @type {?} */
            const urlData = this.parseUrl(url);
            /** @type {?} */
            const data = {};
            /** @type {?} */
            let newBody;
            if (urlData.key) {
                /** @type {?} */
                const foundedIndex = this.mockedItems.findIndex((item) => +item[this._idField] === +urlData.key || ((/** @type {?} */ (item[this._idField]))) === ((/** @type {?} */ (urlData.key))));
                if (foundedIndex !== -1) {
                    newBody = this.mockedItems.get(foundedIndex);
                }
            }
            else {
                /** @type {?} */
                let count = 0;
                /** @type {?} */
                let xTotalCount = 0;
                /** @type {?} */
                const startIndex = (+urlData.page > 1 ? +urlData.page - 1 : 0) * +urlData.limit + 1;
                /** @type {?} */
                const filtredItems = this.mockedItems.filter((item, index) => {
                    if (urlData.searchText && !this.checkWordInData(item, urlData.searchText)) {
                        return false;
                    }
                    else {
                        xTotalCount++;
                    }
                    if (index >= startIndex - 1 && count < +urlData.limit) {
                        count++;
                        return true;
                    }
                    else {
                        return false;
                    }
                });
                newBody = filtredItems.toArray();
                data.headers = new HttpHeaders({
                    'X-Total-Count': xTotalCount.toString()
                });
            }
            data.body = newBody;
            observer.next(data);
        });
    }
    /**
     * @template T
     * @param {?} url
     * @param {?} body
     * @param {?=} options
     * @return {?}
     */
    post(url, body, options) {
        return new Observable(observer => {
            if (!body[this._idField]) {
                body[this._idField] = this.genId();
            }
            this.mockedItems = this.mockedItems.unshift(body);
            /** @type {?} */
            const data = { headers: new HttpHeaders(), body: body };
            observer.next(data);
        });
    }
    /**
     * @template T
     * @param {?} url
     * @param {?} body
     * @param {?=} options
     * @return {?}
     */
    put(url, body, options) {
        return new Observable(observer => {
            /** @type {?} */
            const urlData = this.parseUrl(url);
            /** @type {?} */
            const foundedIndex = this.mockedItems.findIndex((item) => +item[this._idField] === +urlData.key || ((/** @type {?} */ (item[this._idField]))) === ((/** @type {?} */ (urlData.key))));
            if (foundedIndex !== -1) {
                this.mockedItems = this.mockedItems.set(foundedIndex, body);
            }
            /** @type {?} */
            const data = { headers: new HttpHeaders(), body: body };
            observer.next(data);
        });
    }
    /**
     * @template T
     * @param {?} url
     * @param {?=} options
     * @return {?}
     */
    delete(url, options) {
        return new Observable(observer => {
            /** @type {?} */
            const urlData = this.parseUrl(url);
            /** @type {?} */
            const foundedIndex = this.mockedItems.findIndex((item) => +item[this._idField] === +urlData.key || ((/** @type {?} */ (item[this._idField]))) === ((/** @type {?} */ (urlData.key))));
            if (foundedIndex !== -1) {
                this.mockedItems = this.mockedItems.delete(foundedIndex);
            }
            /** @type {?} */
            const data = { headers: new HttpHeaders(), body: true };
            observer.next(data);
        });
    }
    /**
     * @template T
     * @param {?} url
     * @param {?} body
     * @param {?=} options
     * @return {?}
     */
    patch(url, body, options) {
        return new Observable(observer => {
            /** @type {?} */
            const urlData = this.parseUrl(url);
            /** @type {?} */
            const foundedIndex = this.mockedItems.findIndex((item) => +item[this._idField] === +urlData.key || ((/** @type {?} */ (item[this._idField]))) === ((/** @type {?} */ (urlData.key))));
            if (foundedIndex !== -1) {
                /** @type {?} */
                const newBody = this.mockedItems.get(foundedIndex);
                for (const objectKey in body) {
                    if (body[urlData.key] !== undefined) {
                        newBody[urlData.key] = body[urlData.key];
                    }
                }
                this.mockedItems = this.mockedItems.set(foundedIndex, newBody);
            }
            /** @type {?} */
            const data = { headers: new HttpHeaders(), body: body };
            observer.next(data);
        });
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template TModel
 */
class Provider {
    /**
     * @param {?} injector
     * @param {?} factoryModel
     */
    constructor(injector, factoryModel) {
        this.injector = injector;
        this.factoryModel = factoryModel;
        this.delay = 0;
        this.items$ = new BehaviorSubject(List([]));
        this.action$ = new Subject();
        this.create$ = new Subject();
        this.append$ = new Subject();
        this.update$ = new Subject();
        this.patch$ = new Subject();
        this.delete$ = new Subject();
        this.load$ = new Subject();
        this.loadAll$ = new Subject();
        this.actionIsActive$ = new BehaviorSubject(false);
        this.createIsActive$ = new BehaviorSubject(false);
        this.updateIsActive$ = new BehaviorSubject(false);
        this.deleteIsActive$ = new BehaviorSubject(false);
        this.loadIsActive$ = new BehaviorSubject(false);
        this.loadAllIsActive$ = new BehaviorSubject(false);
        this.paginationMeta$ = new BehaviorSubject(new PaginationMeta());
        this.destroy$ = new Subject();
    }
    /**
     * @param {?} data
     * @return {?}
     */
    instanceofFactoryModel(data) {
        return data instanceof this.factoryModel;
    }
    /**
     * @param {?} data
     * @return {?}
     */
    instanceofNestedFactoryModel(data) {
        return (this.factoryModel &&
            this.factoryModel.nested &&
            Object.keys(this.factoryModel.nested).filter(key => data instanceof this.factoryModel.nested[key]).length > 0);
    }
    /**
     * @param {?} data
     * @return {?}
     */
    updateNestedFactoryModel(data) {
        Object.keys(this.factoryModel.nested).forEach(key => this.items$.next(this.items$
            .getValue()
            .map(item => {
            if (item[key]) {
                if (Array.isArray(item[key])) {
                    item[key].forEach((eachItem, index) => {
                        if (eachItem.id === data.id) {
                            item[key][index] = data;
                        }
                    });
                }
                else {
                    item[key] = data;
                }
                this.update(item.id, item, { globalEventIsActive: false });
            }
            return item;
        })
            .toList()));
    }
    /**
     * @param {?} data
     * @return {?}
     */
    deleteNestedFactoryModel(data) {
        Object.keys(this.factoryModel.nested).forEach(key => this.items$.next(this.items$
            .getValue()
            .map(item => {
            if (item[key]) {
                if (Array.isArray(item[key])) {
                    item[key] = item[key].filter(eachItem => eachItem.id !== data.id);
                }
                else {
                    item[key] = undefined;
                }
                this.update(item.id, item, { globalEventIsActive: false });
            }
            return item;
        })
            .toList()));
    }
    // tslint
    /**
     * @param {?} data
     * @param {?} action
     * @param {?=} classTransformOptions
     * @return {?}
     */
    plainToClass(data, action, classTransformOptions) {
        /** @type {?} */
        let model;
        if (!(data instanceof this.factoryModel)) {
            model = (/** @type {?} */ (plainToClassFromExist(new this.factoryModel(), data, classTransformOptions)));
        }
        else {
            model = (/** @type {?} */ (data));
        }
        return model;
    }
    /**
     * @param {?} model
     * @param {?} action
     * @param {?=} classTransformOptions
     * @return {?}
     */
    classToPlain(model, action, classTransformOptions) {
        return classToPlainFromExist(model, {}, classTransformOptions);
    }
    /**
     * @param {?} model
     * @param {?=} classTransformOptions
     * @return {?}
     */
    classToClass(model, classTransformOptions) {
        return classToClass(model, classTransformOptions);
    }
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} data
     * @param {?=} options
     * @return {?}
     */
    action(key, data, options) {
        return of(data).pipe(delay(this.delay));
    }
    /**
     * @template TProviderActionOptions
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    save(model, options) {
        if (model.id === undefined) {
            return this.create(model, options);
        }
        return this.update(model.id, model, options);
    }
    /**
     * @template TProviderActionOptions
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    create(model, options) {
        return of(model).pipe(delay(this.delay));
    }
    /**
     * @template TProviderActionOptions
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    append(model, options) {
        return of(model).pipe(delay(this.delay));
    }
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    update(key, model, options) {
        return of(model).pipe(delay(this.delay));
    }
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    patch(key, model, options) {
        return of(model).pipe(delay(this.delay));
    }
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} options
     * @return {?}
     */
    delete(key, options) {
        /** @type {?} */
        const newModel = this.plainToClass({ id: key }, ProviderActionEnum.Delete, options && options.classTransformOptions ? options.classTransformOptions : undefined);
        return of(newModel).pipe(delay(this.delay));
    }
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} options
     * @return {?}
     */
    load(key, options) {
        /** @type {?} */
        const newModel = this.plainToClass({ id: key }, ProviderActionEnum.Delete, options && options.classTransformOptions ? options.classTransformOptions : undefined);
        return of(newModel).pipe(delay(this.delay));
    }
    /**
     * @template TProviderActionOptions
     * @param {?=} filter
     * @param {?=} options
     * @return {?}
     */
    loadAll(filter, options) {
        /** @type {?} */
        const newModels = [
            this.plainToClass({}, ProviderActionEnum.Delete, options && options.classTransformOptions ? options.classTransformOptions : undefined)
        ];
        return of(newModels).pipe(delay(this.delay));
    }
    /**
     * @param {?} options
     * @return {?}
     */
    calcPaginationMetaByOptions(options) {
        /** @type {?} */
        const paginationMeta = this.paginationMeta$.getValue();
        paginationMeta.totalResults =
            options === undefined || options.paginationMeta === undefined || options.paginationMeta.totalResults === undefined
                ? paginationMeta.totalResults
                : options.paginationMeta.totalResults;
        paginationMeta.perPage =
            options === undefined || options.paginationMeta === undefined || options.paginationMeta.perPage === undefined
                ? paginationMeta.perPage
                : options.paginationMeta.perPage;
        paginationMeta.curPage =
            options === undefined || options.paginationMeta === undefined || options.paginationMeta.curPage === undefined
                ? paginationMeta.curPage
                : options.paginationMeta.curPage;
        paginationMeta.perPage = paginationMeta.perPage === undefined ? 10 : paginationMeta.perPage;
        paginationMeta.curPage = paginationMeta.curPage === undefined ? 1 : paginationMeta.curPage;
        return this.calcPaginationMeta(paginationMeta);
    }
    /**
     * @param {?} newPaginationMeta
     * @return {?}
     */
    calcPaginationMeta(newPaginationMeta) {
        /** @type {?} */
        const paginationMeta = this.paginationMeta$.getValue();
        if (newPaginationMeta.perPage !== undefined) {
            paginationMeta.perPage = newPaginationMeta.perPage;
        }
        if (newPaginationMeta.totalResults !== undefined) {
            paginationMeta.totalResults = newPaginationMeta.totalResults;
            paginationMeta.totalPages = Math.ceil(newPaginationMeta.totalResults / paginationMeta.perPage);
        }
        else {
            if (newPaginationMeta.totalPages !== undefined) {
                paginationMeta.totalResults = undefined;
                paginationMeta.totalPages = newPaginationMeta.totalPages;
            }
        }
        if (newPaginationMeta.curPage !== undefined) {
            paginationMeta.curPage = newPaginationMeta.curPage;
        }
        if (paginationMeta.curPage === 0) {
            paginationMeta.curPage = 1;
        }
        this.paginationMeta$.next(paginationMeta);
        return this.paginationMeta$.getValue();
    }
    /**
     * @return {?}
     */
    reloadAll() { }
    /**
     * @return {?}
     */
    checkFilterAndOptions() { }
    /**
     * @param {?} options
     * @return {?}
     */
    setOptions(options) { }
    /**
     * @return {?}
     */
    getOptions() {
        return {};
    }
    /**
     * @param {?=} items
     * @return {?}
     */
    reconfigItems(items) { }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template TModel
 */
class RestProvider extends Provider {
    /**
     * @param {?} injector
     * @param {?} factoryModel
     * @param {?} options
     */
    constructor(injector, factoryModel, options) {
        super(injector, factoryModel);
        this.injector = injector;
        this.factoryModel = factoryModel;
        this.options = options;
        this.initFakeHttpClient();
        this.init();
    }
    /**
     * @return {?}
     */
    initFakeHttpClient() {
        /** @type {?} */
        const restOptions = this.options === undefined ? undefined : ((/** @type {?} */ (this.options))).restOptions;
        if (restOptions) {
            this.restOptions = restOptions;
        }
        if (!this.restOptions) {
            this.restOptions = {};
            this.restOptions.idField = 'id';
            this.restOptions.pageQueryParam = 'page';
            this.restOptions.limitQueryParam = 'limit';
            this.restOptions.searchTextQueryParam = 'search';
        }
        this.fakeHttpClient = new FakeHttpClient([], this.restOptions);
    }
    /**
     * @return {?}
     */
    init() {
        this.httpClient = this.injector.get(HttpClient);
        this.providerActionHandlers = this.injector.get(RestProviderActionHandlers);
        this.setOptions((/** @type {?} */ (this.options)));
    }
    /**
     * @return {?}
     */
    getOptions() {
        return this.options;
    }
    /**
     * @param {?=} options
     * @return {?}
     */
    setOptions(options) {
        /** @type {?} */
        const filter = options === undefined ? undefined : options.filter;
        options = Object.assign({}, this.options, options);
        /** @type {?} */
        const autoload = options === undefined ? undefined : options.autoload;
        /** @type {?} */
        const pluralName = options === undefined ? undefined : options.pluralName;
        /** @type {?} */
        const name = options === undefined ? undefined : options.name;
        /** @type {?} */
        const apiUrl = options === undefined ? undefined : options.apiUrl;
        /** @type {?} */
        const restOptions = options === undefined ? undefined : options.restOptions;
        if (autoload !== undefined) {
            this.autoload = autoload;
        }
        if (pluralName !== undefined) {
            this.pluralName = pluralName;
        }
        if (name !== undefined) {
            this.name = name;
        }
        if (apiUrl !== undefined) {
            this.apiUrl = apiUrl;
        }
        if (filter !== undefined) {
            this.filter = Object.assign({}, filter);
        }
        options.autoload = this.autoload;
        options.pluralName = this.pluralName;
        options.name = this.name;
        options.apiUrl = this.apiUrl;
        options.filter = Object.assign({}, this.filter);
        this.options = options;
        this.calcPaginationMetaByOptions(options);
        if (this.autoload !== false) {
            this._loadAll(this.filter, this.prevOptions)
                .pipe(first())
                .subscribe();
        }
    }
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} data
     * @param {?=} options
     * @return {?}
     */
    action(key, data, options) {
        this.actionIsActive$.next(true);
        /** @type {?} */
        const useDefault = false;
        // (options && options.useFakeHttpClient === true) || this.httpClient instanceof FakeHttpClient;
        /** @type {?} */
        const errors = validateSync(data, options && options.classValidatorOptions ? options.classValidatorOptions : { validationError: { target: false } });
        if (errors.length > 0 && (!options || options.globalEventIsActive !== false)) {
            return throwError(new ValidatorError(errors));
        }
        /** @type {?} */
        const optionsList = [{ actionOptions: options }, (/** @type {?} */ (this.options))];
        /** @type {?} */
        const requestUrl = this.providerActionHandlers.getRequestUrl(key, data, optionsList, ProviderActionEnum.Action, useDefault);
        /** @type {?} */
        const requestOptions = this.providerActionHandlers.getRequestOptions(key, data, optionsList, ProviderActionEnum.Action, useDefault);
        /** @type {?} */
        let request = this.providerActionHandlers.getRequest(requestUrl, data, requestOptions, optionsList, ProviderActionEnum.Action, useDefault);
        if (!request) {
            request = this.httpClient.post(requestUrl, data, requestOptions);
        }
        return request.pipe(map(responseData => this.providerActionHandlers.getResponseData(responseData, optionsList, ProviderActionEnum.Action, useDefault)), map(actionData => {
            if (options === undefined ||
                options.globalEventIsActive !== false ||
                options.globalEventIsActiveResolver !== undefined) {
                if (options === undefined ||
                    options.globalEventIsActiveResolver === undefined ||
                    options.globalEventIsActiveResolver({
                        actionKey: key,
                        requestData: data,
                        responseData: actionData
                    }, ProviderActionEnum.Action)) {
                    this.action$.next({
                        actionKey: key,
                        requestData: data,
                        responseData: actionData
                    });
                }
            }
            this.actionIsActive$.next(false);
            return actionData;
        }));
    }
    /**
     * @template TProviderActionOptions
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    create(model, options) {
        this.createIsActive$.next(true);
        /** @type {?} */
        const useDefault = (options && options.useFakeHttpClient === true) || this.httpClient instanceof FakeHttpClient;
        model = this.plainToClass(model, ProviderActionEnum.Create, options && options.classTransformOptions ? options.classTransformOptions : undefined);
        /** @type {?} */
        const errors = validateSync(model, options && options.classValidatorOptions ? options.classValidatorOptions : { validationError: { target: false } });
        if (errors.length > 0 && (!options || options.globalEventIsActive !== false)) {
            return throwError(new ValidatorError(errors));
        }
        /** @type {?} */
        const optionsList = [{ actionOptions: options }, (/** @type {?} */ (this.options))];
        /** @type {?} */
        const isCreate = this.providerActionHandlers.getRequestCreateType(optionsList, ProviderActionEnum.Create, useDefault) === 'create';
        /** @type {?} */
        let object;
        if (isCreate === true) {
            object = this.classToPlain(model, ProviderActionEnum.Create, options && options.classTransformOptions ? options.classTransformOptions : undefined);
        }
        else {
            object = this.classToPlain(model, ProviderActionEnum.Append, options && options.classTransformOptions ? options.classTransformOptions : undefined);
        }
        /** @type {?} */
        const requestUrl = this.providerActionHandlers.getRequestUrl(undefined, object, optionsList, isCreate ? ProviderActionEnum.Create : ProviderActionEnum.Append, useDefault);
        /** @type {?} */
        const requestOptions = this.providerActionHandlers.getRequestOptions(undefined, object, optionsList, isCreate ? ProviderActionEnum.Create : ProviderActionEnum.Append, useDefault);
        /** @type {?} */
        let request = this.providerActionHandlers.getRequest(requestUrl, object, requestOptions, optionsList, isCreate ? ProviderActionEnum.Create : ProviderActionEnum.Append, useDefault);
        if (!request) {
            request = (options === undefined || options.useFakeHttpClient !== true
                ? this.httpClient
                : this.fakeHttpClient).post(requestUrl, object, requestOptions);
        }
        return request.pipe(map(responseData => this.providerActionHandlers.getResponseData(responseData, optionsList, isCreate ? ProviderActionEnum.Create : ProviderActionEnum.Append, useDefault)), map(createdItem => {
            /** @type {?} */
            let createdModel;
            if (isCreate === true) {
                createdModel = this.plainToClass(createdItem, ProviderActionEnum.Create, options && options.classTransformOptions ? options.classTransformOptions : undefined);
                this.items$.next(this.items$.getValue().unshift(createdModel));
            }
            else {
                createdModel = this.plainToClass(createdItem, ProviderActionEnum.Append, options && options.classTransformOptions ? options.classTransformOptions : undefined);
                this.items$.next(this.items$.getValue().push(createdModel));
            }
            /** @type {?} */
            const paginationMeta = this.paginationMeta$.getValue();
            this.calcPaginationMeta({ totalResults: paginationMeta.totalResults + 1 });
            this.reconfigItems();
            if (options === undefined ||
                options.globalEventIsActive !== false ||
                options.globalEventIsActiveResolver !== undefined) {
                if (options === undefined ||
                    options.globalEventIsActiveResolver === undefined ||
                    options.globalEventIsActiveResolver(createdModel, isCreate ? ProviderActionEnum.Create : ProviderActionEnum.Append)) {
                    (isCreate ? this.create$ : this.append$).next(createdModel);
                }
            }
            this.createIsActive$.next(false);
            return createdModel;
        }));
    }
    /**
     * @private
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?} model
     * @param {?} isUpdate
     * @param {?=} options
     * @return {?}
     */
    _update(key, model, isUpdate, options) {
        this.updateIsActive$.next(true);
        /** @type {?} */
        const useDefault = (options && options.useFakeHttpClient === true) || this.httpClient instanceof FakeHttpClient;
        model = this.plainToClass(model, ProviderActionEnum.Update, options && options.classTransformOptions ? options.classTransformOptions : undefined);
        /** @type {?} */
        const errors = validateSync(model, options && options.classValidatorOptions ? options.classValidatorOptions : { validationError: { target: false } });
        if (errors.length > 0 && (!options || options.globalEventIsActive !== false)) {
            return throwError(new ValidatorError(errors));
        }
        /** @type {?} */
        let object;
        /** @type {?} */
        const optionsList = [{ actionOptions: options }, (/** @type {?} */ (this.options))];
        /** @type {?} */
        let request;
        if (isUpdate === true) {
            object = this.classToPlain(model, ProviderActionEnum.Update, options && options.classTransformOptions ? options.classTransformOptions : undefined);
            /** @type {?} */
            const requestUrl = this.providerActionHandlers.getRequestUrl(key, object, optionsList, ProviderActionEnum.Update, useDefault);
            /** @type {?} */
            const requestOptions = this.providerActionHandlers.getRequestOptions(key, object, optionsList, ProviderActionEnum.Update, useDefault);
            request = this.providerActionHandlers.getRequest(requestUrl, object, requestOptions, optionsList, ProviderActionEnum.Update, useDefault);
            if (!request) {
                request = (options === undefined || options.useFakeHttpClient !== true
                    ? this.httpClient
                    : this.fakeHttpClient).put(requestUrl, object, requestOptions);
            }
        }
        else {
            object = this.classToPlain(model, ProviderActionEnum.Patch, options && options.classTransformOptions ? options.classTransformOptions : undefined);
            /** @type {?} */
            const requestUrl = this.providerActionHandlers.getRequestUrl(key, object, optionsList, ProviderActionEnum.Patch, useDefault);
            /** @type {?} */
            const requestOptions = this.providerActionHandlers.getRequestOptions(key, object, optionsList, ProviderActionEnum.Patch, useDefault);
            request = this.providerActionHandlers.getRequest(requestUrl, object, requestOptions, optionsList, ProviderActionEnum.Patch, useDefault);
            if (!request) {
                request = this.httpClient.patch(requestUrl, object, requestOptions);
            }
        }
        return request.pipe(map(responseData => this.providerActionHandlers.getResponseData(responseData, optionsList, isUpdate ? ProviderActionEnum.Update : ProviderActionEnum.Patch, useDefault)), map(updatedItem => {
            /** @type {?} */
            const updatedModel = this.plainToClass(updatedItem, isUpdate ? ProviderActionEnum.Update : ProviderActionEnum.Patch);
            /** @type {?} */
            const index = this.items$.getValue().findIndex(eachModel => eachModel.id === key);
            if (index !== -1) {
                this.reconfigItems(this.items$.getValue().set(index, updatedModel));
            }
            if (options === undefined ||
                options.globalEventIsActive !== false ||
                options.globalEventIsActiveResolver !== undefined) {
                if (options === undefined ||
                    options.globalEventIsActiveResolver === undefined ||
                    options.globalEventIsActiveResolver(updatedModel, isUpdate ? ProviderActionEnum.Update : ProviderActionEnum.Patch)) {
                    (isUpdate ? this.update$ : this.patch$).next(updatedModel);
                }
            }
            this.updateIsActive$.next(false);
            return updatedModel;
        }));
    }
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    update(key, model, options) {
        return this._update(key, model, true, options);
    }
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    patch(key, model, options) {
        return this._update(key, model, false, options);
    }
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} options
     * @return {?}
     */
    load(key, options) {
        this.loadIsActive$.next(true);
        /** @type {?} */
        const useDefault = (options && options.useFakeHttpClient === true) || this.httpClient instanceof FakeHttpClient;
        /** @type {?} */
        const optionsList = [{ actionOptions: options }, (/** @type {?} */ (this.options))];
        /** @type {?} */
        const requestUrl = this.providerActionHandlers.getRequestUrl(key, undefined, optionsList, ProviderActionEnum.Load, useDefault);
        /** @type {?} */
        const requestOptions = this.providerActionHandlers.getRequestOptions(key, undefined, optionsList, ProviderActionEnum.Load, useDefault);
        /** @type {?} */
        let request = this.providerActionHandlers.getRequest(requestUrl, undefined, requestOptions, optionsList, ProviderActionEnum.Load, useDefault);
        if (!request) {
            request = (options === undefined || options.useFakeHttpClient !== true
                ? this.httpClient
                : this.fakeHttpClient).get(requestUrl, requestOptions);
        }
        return request.pipe(map(responseData => this.providerActionHandlers.getResponseData(responseData, optionsList, ProviderActionEnum.Load, useDefault)), map(loadedItem => {
            /** @type {?} */
            const loadedModel = this.plainToClass(loadedItem, ProviderActionEnum.Load, options && options.classTransformOptions ? options.classTransformOptions : undefined);
            /** @type {?} */
            const index = this.items$.getValue().findIndex(eachModel => eachModel.id === key);
            if (index !== -1) {
                this.items$.next(this.items$.getValue().set(index, loadedModel));
            }
            if (options === undefined ||
                options.globalEventIsActive !== false ||
                options.globalEventIsActiveResolver !== undefined) {
                if (options === undefined ||
                    options.globalEventIsActiveResolver === undefined ||
                    options.globalEventIsActiveResolver(loadedModel, ProviderActionEnum.Load)) {
                    this.load$.next(loadedModel);
                }
            }
            this.loadIsActive$.next(false);
            return loadedModel;
        }));
    }
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} options
     * @return {?}
     */
    delete(key, options) {
        this.deleteIsActive$.next(true);
        /** @type {?} */
        const useDefault = (options && options.useFakeHttpClient === true) || this.httpClient instanceof FakeHttpClient;
        /** @type {?} */
        const optionsList = [{ actionOptions: options }, (/** @type {?} */ (this.options))];
        /** @type {?} */
        const requestUrl = this.providerActionHandlers.getRequestUrl(key, undefined, optionsList, ProviderActionEnum.Delete, useDefault);
        /** @type {?} */
        const requestOptions = this.providerActionHandlers.getRequestOptions(key, undefined, optionsList, ProviderActionEnum.Delete, useDefault);
        /** @type {?} */
        let request = this.providerActionHandlers.getRequest(requestUrl, undefined, requestOptions, optionsList, ProviderActionEnum.Delete, useDefault);
        if (!request) {
            request = (options === undefined || options.useFakeHttpClient !== true
                ? this.httpClient
                : this.fakeHttpClient).delete(requestUrl, requestOptions);
        }
        return request.pipe(map(responseData => this.providerActionHandlers.getResponseData(responseData, optionsList, ProviderActionEnum.Delete, useDefault)), map(deleted => {
            /** @type {?} */
            const index = this.items$.getValue().findIndex(eachModel => eachModel.id === key);
            /** @type {?} */
            const deletedModel = this.items$.getValue().get(index);
            if (index !== -1) {
                this.items$.next(this.items$.getValue().delete(index));
                /** @type {?} */
                const paginationMeta = this.paginationMeta$.getValue();
                /** @type {?} */
                const newPaginationMeta = this.calcPaginationMeta({
                    totalResults: paginationMeta.totalResults === 0 ? 0 : paginationMeta.totalResults - 1
                });
                this.reconfigItems();
            }
            if (options === undefined ||
                options.globalEventIsActive !== false ||
                options.globalEventIsActiveResolver !== undefined) {
                if (options === undefined ||
                    options.globalEventIsActiveResolver === undefined ||
                    options.globalEventIsActiveResolver(deletedModel, ProviderActionEnum.Delete)) {
                    this.delete$.next(deletedModel);
                }
            }
            this.deleteIsActive$.next(false);
            return deletedModel;
        }));
    }
    /**
     * @return {?}
     */
    reloadAll() {
        this._loadAll(this.filter, this.prevOptions)
            .pipe(first())
            .subscribe();
    }
    /**
     * @template TProviderActionOptions
     * @param {?=} filter
     * @param {?=} options
     * @return {?}
     */
    loadAll(filter, options) {
        filter = Object.assign({}, filter);
        /** @type {?} */
        const searchText = filter[this.restOptions.searchTextQueryParam] || filter.searchText;
        delete filter.searchText;
        this.calcPaginationMeta({
            curPage: filter[this.restOptions.pageQueryParam] || filter.curPage,
            perPage: filter[this.restOptions.limitQueryParam] || filter.perPage
        });
        delete filter.curPage;
        delete filter.perPage;
        filter[this.restOptions.searchTextQueryParam] = searchText;
        return this._loadAll(filter, options);
    }
    /**
     * @private
     * @template TProviderActionOptions
     * @param {?=} filter
     * @param {?=} options
     * @return {?}
     */
    _loadAll(filter, options) {
        this.loadAllIsActive$.next(true);
        /** @type {?} */
        const useDefault = (options && options.useFakeHttpClient === true) || this.httpClient instanceof FakeHttpClient;
        /** @type {?} */
        const optionsList = [{ actionOptions: options }, (/** @type {?} */ (this.options))];
        /** @type {?} */
        const paginationMeta = this.paginationMeta$.getValue();
        this.filter = filter ? Object.assign({}, filter) : {};
        for (const key in this.filter) {
            if (this.filter.hasOwnProperty(key) && !this.filter[key]) {
                delete this.filter[key];
            }
        }
        this.prevOptions = options;
        /** @type {?} */
        let requestUrl = this.providerActionHandlers.getRequestUrl(undefined, this.filter, optionsList, ProviderActionEnum.LoadAll, useDefault);
        this.filter[this.restOptions.pageQueryParam] = paginationMeta.curPage;
        this.filter[this.restOptions.limitQueryParam] = paginationMeta.perPage;
        requestUrl = this.providerActionHandlers.getRequestQuery(requestUrl, this.filter, optionsList, ProviderActionEnum.LoadAll, useDefault);
        /** @type {?} */
        const requestOptions = this.providerActionHandlers.getRequestOptions(undefined, this.filter, optionsList, ProviderActionEnum.LoadAll, useDefault);
        /** @type {?} */
        let request = this.providerActionHandlers.getRequest(requestUrl, undefined, requestOptions, optionsList, ProviderActionEnum.LoadAll, useDefault);
        if (!request) {
            request = (options === undefined || options.useFakeHttpClient !== true
                ? this.httpClient
                : this.fakeHttpClient).get(requestUrl, requestOptions);
        }
        return request.pipe(map(responseData => {
            /** @type {?} */
            const responsePaginationMeta = this.providerActionHandlers.getResponsePaginationMeta(responseData, optionsList, ProviderActionEnum.LoadAll, useDefault);
            if (responsePaginationMeta) {
                this.calcPaginationMeta(Object.assign({ curPage: paginationMeta.curPage, perPage: paginationMeta.perPage }, responsePaginationMeta));
            }
            else {
                this.calcPaginationMeta({
                    curPage: paginationMeta.curPage,
                    perPage: paginationMeta.perPage
                });
            }
            return this.providerActionHandlers.getResponseData(responseData, optionsList, ProviderActionEnum.LoadAll, useDefault);
        }), map(loadedItems => {
            /** @type {?} */
            const newPaginationMeta = this.paginationMeta$.getValue();
            /** @type {?} */
            let loadedModels = loadedItems === undefined
                ? []
                : loadedItems.map(loadedItem => this.plainToClass(loadedItem, ProviderActionEnum.LoadAll, options && options.classTransformOptions ? options.classTransformOptions : undefined));
            if (this.options.infinity && newPaginationMeta.curPage > 1) {
                /** @type {?} */
                const items = this.items$.getValue().toArray();
                loadedModels = [...items, ...loadedModels];
            }
            this.reconfigItems(List(loadedModels));
            if (options === undefined ||
                options.globalEventIsActive !== false ||
                options.globalEventIsActiveResolver !== undefined) {
                if (options === undefined ||
                    options.globalEventIsActiveResolver === undefined ||
                    options.globalEventIsActiveResolver(loadedModels, ProviderActionEnum.LoadAll)) {
                    this.loadAll$.next(loadedModels);
                }
            }
            this.loadAllIsActive$.next(false);
            return loadedModels;
        }));
    }
    /**
     * @param {?=} items
     * @return {?}
     */
    reconfigItems(items) {
        /** @type {?} */
        const newPaginationMeta = this.paginationMeta$.getValue();
        if (newPaginationMeta.perPage === undefined) {
            throw new ProviderError('Not set perPage count');
        }
        if (this.options.infinity && newPaginationMeta.curPage > 1) {
            if (items) {
                this.items$.next(items);
            }
            else {
                this.items$.next(this.items$.getValue());
            }
        }
        else {
            if (items) {
                this.items$.next(List(items.take(newPaginationMeta.perPage)));
            }
            else {
                this.items$.next(List(this.items$.getValue().take(newPaginationMeta.perPage)));
            }
        }
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template TModel
 */
class MockProvider extends RestProvider {
    /**
     * @param {?} injector
     * @param {?} factoryModel
     * @param {?} options
     */
    constructor(injector, factoryModel, options) {
        super(injector, factoryModel, options);
        this.injector = injector;
        this.factoryModel = factoryModel;
        this.options = options;
    }
    /**
     * @return {?}
     */
    init() {
        this.httpClient = this.fakeHttpClient;
        this.providerActionHandlers = this.injector.get(MockProviderActionHandlers);
        this.setOptions((/** @type {?} */ (this.options)));
    }
    /**
     * @param {?=} options
     * @return {?}
     */
    setOptions(options) {
        if (this.apiUrl === undefined) {
            this.apiUrl = 'http://fake/api';
        }
        if (this.pluralName === undefined) {
            this.pluralName = 'entities';
        }
        if (this.name === undefined) {
            this.name = 'entity';
        }
        if (options && options.items) {
            ((/** @type {?} */ (this.httpClient))).setItems(options.items);
        }
        super.setOptions(options);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template TModel
 */
class Repository {
    /**
     * @param {?} injector
     * @param {?=} factoryModel
     * @param {?=} factoryMockProvider
     * @param {?=} factoryRestProvider
     */
    constructor(injector, factoryModel, factoryMockProvider, factoryRestProvider) {
        this.injector = injector;
        this.factoryModel = factoryModel;
        this.factoryMockProvider = factoryMockProvider;
        this.factoryRestProvider = factoryRestProvider;
        this.mockProviders = [];
        this.restProviders = [];
        this._mockProviderIsActive = false;
        this._restProviderIsActive = false;
        this.destroyed$ = new Subject();
    }
    /**
     * @return {?}
     */
    get provider() {
        if (this._mockProviderIsActive && this.mockProvider !== undefined) {
            return this.mockProvider;
        }
        if (this._restProviderIsActive && this.restProvider !== undefined) {
            return this.restProvider;
        }
        return new Provider(this.injector, this.factoryModel);
    }
    /**
     * @return {?}
     */
    get paginationMeta$() {
        return this.provider.paginationMeta$;
    }
    /**
     * @return {?}
     */
    get providers() {
        return [this.restProvider, ...this.restProviders, this.mockProvider, ...this.mockProviders];
    }
    /**
     * @return {?}
     */
    get items() {
        return this.provider.items$.getValue().toArray();
    }
    /**
     * @return {?}
     */
    get items$() {
        return this.provider.items$.pipe(map(items => items.toArray()));
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.destroyed$.next(true);
        this.destroyed$.complete();
    }
    /**
     * @template TNewModel
     * @param {?=} newFactoryModel
     * @param {?=} newFactoryMockProvider
     * @param {?=} newFactoryRestProvider
     * @return {?}
     */
    fork(newFactoryModel, newFactoryMockProvider, newFactoryRestProvider) {
        if (this.root !== undefined) {
            return this.root.fork(newFactoryModel, newFactoryMockProvider, newFactoryRestProvider);
        }
        /** @type {?} */
        const newInstance = new Repository(this.injector, newFactoryModel ? newFactoryModel : ((/** @type {?} */ (this.factoryModel))), newFactoryMockProvider ? newFactoryMockProvider : ((/** @type {?} */ (this.factoryMockProvider))), newFactoryRestProvider ? newFactoryRestProvider : ((/** @type {?} */ (this.factoryRestProvider))));
        newInstance.root = (/** @type {?} */ (this));
        return newInstance;
    }
    /**
     * @template TMockProviderOptions
     * @param {?=} options
     * @return {?}
     */
    useMock(options) {
        this._restProviderIsActive = false;
        this._mockProviderIsActive = true;
        if (this.mockProvider === undefined && options !== undefined) {
            this.mockProvider = (/** @type {?} */ ((this.factoryMockProvider
                ? new this.factoryMockProvider(this.injector, this.factoryModel, options)
                : new MockProvider(this.injector, this.factoryModel, options))));
            this.mockProvider.name = 'Mock#' + this.root.mockProviders.length.toString();
            if (this.root !== undefined) {
                this.root.subscribeToProvider(this.mockProvider);
                this.root.mockProviders.push(this.mockProvider);
            }
        }
        else {
            this.mockProvider.name = 'Mock#root';
            this.mockProvider.setOptions(options);
            this.subscribeToProvider(this.mockProvider);
        }
    }
    /**
     * @template TMockProviderOptions
     * @param {?=} options
     * @return {?}
     */
    useRest(options) {
        // tslint:disable-line
        this._restProviderIsActive = true;
        this._mockProviderIsActive = false;
        if (this.restProvider === undefined && options !== undefined) {
            this.restProvider = (/** @type {?} */ ((this.factoryRestProvider
                ? new this.factoryRestProvider(this.injector, this.factoryModel, options)
                : new RestProvider(this.injector, this.factoryModel, options))));
            this.restProvider.name = 'Rest#' + this.root.restProviders.length.toString();
            if (this.root !== undefined) {
                this.root.subscribeToProvider(this.restProvider);
                this.root.restProviders.push(this.restProvider);
            }
        }
        else {
            this.restProvider.name = 'Rest#root';
            this.restProvider.setOptions(options);
            this.subscribeToProvider(this.restProvider);
        }
    }
    /**
     * @param {?} provider
     * @return {?}
     */
    subscribeToProvider(provider) {
        provider.create$.pipe(takeUntil(this.destroyed$)).subscribe(item => {
            this.providers.forEach(eachProvider => {
                /** @type {?} */
                const options = eachProvider ? eachProvider.getOptions() : undefined;
                if (eachProvider !== undefined &&
                    eachProvider.name !== provider.name &&
                    eachProvider.instanceofFactoryModel(item) &&
                    (options === undefined ||
                        (options.globalEventResolver === undefined || options.globalEventResolver(item, ProviderActionEnum.Create)))) {
                    eachProvider
                        .create(item, { useFakeHttpClient: true, globalEventIsActive: false })
                        .pipe(first())
                        .subscribe();
                }
            });
        });
        provider.append$.pipe(takeUntil(this.destroyed$)).subscribe(item => {
            this.providers.forEach(eachProvider => {
                /** @type {?} */
                const options = eachProvider ? eachProvider.getOptions() : undefined;
                if (eachProvider !== undefined &&
                    eachProvider.name !== provider.name &&
                    eachProvider.instanceofFactoryModel(item) &&
                    (options === undefined ||
                        (options.globalEventResolver === undefined || options.globalEventResolver(item, ProviderActionEnum.Create)))) {
                    eachProvider
                        .append(item, { useFakeHttpClient: true, globalEventIsActive: false })
                        .pipe(first())
                        .subscribe();
                }
            });
        });
        provider.update$.pipe(takeUntil(this.destroyed$)).subscribe(item => {
            this.providers.forEach(eachProvider => {
                /** @type {?} */
                const options = eachProvider ? eachProvider.getOptions() : undefined;
                if (eachProvider !== undefined && eachProvider.name !== provider.name) {
                    if (eachProvider.instanceofFactoryModel(item) &&
                        (options === undefined ||
                            (options.globalEventResolver === undefined ||
                                options.globalEventResolver(item, ProviderActionEnum.Update)))) {
                        eachProvider
                            .update(item.id, item, { useFakeHttpClient: true, globalEventIsActive: false })
                            .pipe(first())
                            .subscribe();
                    }
                    if (eachProvider.instanceofNestedFactoryModel(item)) {
                        eachProvider.updateNestedFactoryModel(item);
                    }
                }
            });
        });
        provider.patch$.pipe(takeUntil(this.destroyed$)).subscribe(item => {
            this.providers.forEach(eachProvider => {
                /** @type {?} */
                const options = eachProvider ? eachProvider.getOptions() : undefined;
                if (eachProvider !== undefined && eachProvider.name !== provider.name) {
                    if (eachProvider.instanceofFactoryModel(item) &&
                        (options === undefined ||
                            (options.globalEventResolver === undefined ||
                                options.globalEventResolver(item, ProviderActionEnum.Patch)))) {
                        eachProvider
                            .patch(item.id, item, { useFakeHttpClient: true, globalEventIsActive: false })
                            .pipe(first())
                            .subscribe();
                    }
                }
            });
        });
        provider.delete$.pipe(takeUntil(this.destroyed$)).subscribe(item => {
            this.providers.forEach(eachProvider => {
                /** @type {?} */
                const options = eachProvider ? eachProvider.getOptions() : undefined;
                if (eachProvider !== undefined && eachProvider.name !== provider.name) {
                    if (eachProvider.instanceofFactoryModel(item) &&
                        (options === undefined ||
                            (options.globalEventResolver === undefined ||
                                options.globalEventResolver(item, ProviderActionEnum.Delete)))) {
                        eachProvider
                            .delete(item.id, { useFakeHttpClient: true, globalEventIsActive: false })
                            .pipe(first())
                            .subscribe();
                    }
                    if (eachProvider.instanceofNestedFactoryModel(item)) {
                        eachProvider.deleteNestedFactoryModel(item);
                    }
                }
            });
        });
        provider.load$.pipe(takeUntil(this.destroyed$)).subscribe(item => {
            this.providers.forEach(eachProvider => {
                /** @type {?} */
                const options = eachProvider ? eachProvider.getOptions() : undefined;
                if (eachProvider !== undefined &&
                    eachProvider.name !== provider.name &&
                    eachProvider.instanceofFactoryModel(item) &&
                    (options === undefined ||
                        (options.globalEventResolver === undefined || options.globalEventResolver(item, ProviderActionEnum.Update)))) {
                    eachProvider
                        .update(item.id, item, { useFakeHttpClient: true, globalEventIsActive: false })
                        .pipe(first())
                        .subscribe();
                }
            });
        });
        provider.loadAll$.pipe(takeUntil(this.destroyed$)).subscribe(items => {
            items.forEach(item => this.providers.forEach(eachProvider => {
                /** @type {?} */
                const options = eachProvider ? eachProvider.getOptions() : undefined;
                if (eachProvider !== undefined &&
                    eachProvider.name !== provider.name &&
                    eachProvider.instanceofFactoryModel(item) &&
                    (options === undefined ||
                        (options.globalEventResolver === undefined ||
                            options.globalEventResolver(item, ProviderActionEnum.Update)))) {
                    eachProvider
                        .update(item.id, item, { useFakeHttpClient: true, globalEventIsActive: false })
                        .pipe(first())
                        .subscribe();
                }
            }));
        });
    }
    /**
     * @param {?} options
     * @return {?}
     */
    setOptions(options) {
        this.provider.setOptions(options);
    }
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} data
     * @param {?=} options
     * @return {?}
     */
    action(key, data, options) {
        return this.provider.action(key, data, options).pipe(first());
    }
    /**
     * @template TProviderActionOptions
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    save(model, options) {
        return this.provider.save(model, options).pipe(first());
    }
    /**
     * @template TProviderActionOptions
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    create(model, options) {
        return this.provider.create(model, options).pipe(first());
    }
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    update(key, model, options) {
        return this.provider.update(key, model, options).pipe(first());
    }
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    patch(key, model, options) {
        return this.provider.patch(key, model, options).pipe(first());
    }
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} options
     * @return {?}
     */
    load(key, options) {
        return this.provider.load(key, options).pipe(first());
    }
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} options
     * @return {?}
     */
    delete(key, options) {
        return this.provider.delete(key, options).pipe(first());
    }
    /**
     * @return {?}
     */
    reloadAll() {
        this.provider.reloadAll();
    }
    /**
     * @template TProviderActionOptions
     * @param {?=} filter
     * @param {?=} options
     * @return {?}
     */
    loadAll(filter, options) {
        return this.provider.loadAll(filter, options);
    }
    /**
     * @param {?} model
     * @return {?}
     */
    clone(model) {
        return this.provider.classToClass(model);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DynamicRepository extends Repository {
    /**
     * @param {?} injector
     */
    constructor(injector) {
        super(injector, Model);
        this.injector = injector;
    }
}
DynamicRepository.decorators = [
    { type: Injectable }
];
/** @nocollapse */
DynamicRepository.ctorParameters = () => [
    { type: Injector }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgxRepositoryModule {
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: NgxRepositoryModule,
            providers: []
        };
    }
}
NgxRepositoryModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                providers: [RestProviderActionHandlers, MockProviderActionHandlers, DynamicRepository]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NgxRepositoryModule, ProviderActionEnum, ProviderError, ValidatorError, Model, PaginationMeta, MockProviderActionHandlers, MockProvider, Provider, RestProviderActionHandlers, RestProvider, DynamicRepository, Repository, FakeHttpClient };

//# sourceMappingURL=ngx-repository.js.map