import { CommonModule } from '@angular/common';
import { Injectable, Injector, NgModule } from '@angular/core';
import { validateSync } from 'class-validator';
import { __extends, __spread, __assign } from 'tslib';
import { HttpHeaders, HttpClient } from '@angular/common/http';
import * as parseImported from 'url-parse';
import { plainToClassFromExist, classToClass, classToPlainFromExist } from 'class-transformer';
import { List } from 'immutable';
import { Observable, BehaviorSubject, Subject, of, throwError } from 'rxjs';
import { delay, first, map, takeUntil } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {number} */
var ProviderActionEnum = {
    Action: 0,
    Create: 1,
    Append: 2,
    Update: 3,
    Patch: 4,
    Delete: 5,
    Load: 6,
    LoadAll: 7,
};
ProviderActionEnum[ProviderActionEnum.Action] = 'Action';
ProviderActionEnum[ProviderActionEnum.Create] = 'Create';
ProviderActionEnum[ProviderActionEnum.Append] = 'Append';
ProviderActionEnum[ProviderActionEnum.Update] = 'Update';
ProviderActionEnum[ProviderActionEnum.Patch] = 'Patch';
ProviderActionEnum[ProviderActionEnum.Delete] = 'Delete';
ProviderActionEnum[ProviderActionEnum.Load] = 'Load';
ProviderActionEnum[ProviderActionEnum.LoadAll] = 'LoadAll';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var PaginationMeta = /** @class */ (function () {
    function PaginationMeta(data) {
        plainToClassFromExist(this, data);
    }
    return PaginationMeta;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var RestProviderActionHandlers = /** @class */ (function () {
    function RestProviderActionHandlers() {
    }
    /**
     * @param {?} requestUrl
     * @param {?} data
     * @param {?} requestOptions
     * @param {?} optionsList
     * @param {?} action
     * @param {?=} useDefault
     * @return {?}
     */
    RestProviderActionHandlers.prototype.getRequest = /**
     * @param {?} requestUrl
     * @param {?} data
     * @param {?} requestOptions
     * @param {?} optionsList
     * @param {?} action
     * @param {?=} useDefault
     * @return {?}
     */
    function (requestUrl, data, requestOptions, optionsList, action, useDefault) {
        /** @type {?} */
        var request;
        if (useDefault !== true) {
            optionsList.forEach(function (eachOptions) {
                if (eachOptions !== undefined &&
                    eachOptions.actionOptions !== undefined &&
                    eachOptions.actionOptions.request !== undefined) {
                    request = eachOptions.actionOptions.request;
                    return request;
                }
            });
        }
        if (request !== undefined) {
            return request(requestUrl, data, requestOptions);
        }
        return undefined;
    };
    /**
     * @param {?} data
     * @param {?} optionsList
     * @param {?} action
     * @param {?=} useDefault
     * @return {?}
     */
    RestProviderActionHandlers.prototype.getResponseData = /**
     * @param {?} data
     * @param {?} optionsList
     * @param {?} action
     * @param {?=} useDefault
     * @return {?}
     */
    function (data, optionsList, action, useDefault) {
        /** @type {?} */
        var responseData;
        if (useDefault !== true) {
            optionsList.forEach(function (eachOptions) {
                if (eachOptions !== undefined &&
                    eachOptions.actionOptions !== undefined &&
                    eachOptions.actionOptions.responseData !== undefined) {
                    responseData = eachOptions.actionOptions.responseData(data, action);
                    return responseData;
                }
            });
        }
        if (responseData !== undefined) {
            return responseData;
        }
        return data.body;
    };
    /**
     * @param {?} data
     * @param {?} optionsList
     * @param {?} action
     * @param {?=} useDefault
     * @return {?}
     */
    RestProviderActionHandlers.prototype.getResponsePaginationMeta = /**
     * @param {?} data
     * @param {?} optionsList
     * @param {?} action
     * @param {?=} useDefault
     * @return {?}
     */
    function (data, optionsList, action, useDefault) {
        /** @type {?} */
        var paginationMeta;
        if (useDefault !== true) {
            optionsList.forEach(function (eachOptions) {
                if (eachOptions !== undefined &&
                    eachOptions.actionOptions !== undefined &&
                    eachOptions.actionOptions.responsePaginationMeta !== undefined) {
                    paginationMeta = eachOptions.actionOptions.responsePaginationMeta(data, action);
                    return paginationMeta;
                }
            });
        }
        if (paginationMeta !== undefined) {
            return paginationMeta;
        }
        paginationMeta = new PaginationMeta();
        /** @type {?} */
        var xTotalCount = NaN;
        /** @type {?} */
        var keys = data.headers ? (data.headers.keys ? data.headers.keys() : data.headers) : [];
        keys.forEach(function (key) {
            if (key.toLowerCase() === 'x-total-count') {
                xTotalCount = +data.headers.get(key);
            }
        });
        paginationMeta.totalResults = isNaN(xTotalCount) ? 10000 : xTotalCount;
        return paginationMeta;
    };
    /**
     * @param {?} currentUrl
     * @param {?} filter
     * @param {?} optionsList
     * @param {?} action
     * @param {?=} useDefault
     * @return {?}
     */
    RestProviderActionHandlers.prototype.getRequestQuery = /**
     * @param {?} currentUrl
     * @param {?} filter
     * @param {?} optionsList
     * @param {?} action
     * @param {?=} useDefault
     * @return {?}
     */
    function (currentUrl, filter, optionsList, action, useDefault) {
        /** @type {?} */
        var query;
        if (useDefault !== true) {
            optionsList.forEach(function (eachOptions) {
                if (eachOptions !== undefined &&
                    eachOptions.actionOptions !== undefined &&
                    eachOptions.actionOptions.requestQuery !== undefined) {
                    query = eachOptions.actionOptions.requestQuery(currentUrl, filter, action);
                    return query;
                }
            });
        }
        if (query !== undefined) {
            return query;
        }
        for (var key in filter) {
            if (filter.hasOwnProperty(key)) {
                /** @type {?} */
                var value = filter[key];
                if (!currentUrl.includes("?" + key + "=") && !currentUrl.includes("&" + key + "=")) {
                    currentUrl = currentUrl + ((currentUrl.indexOf('?') === -1 ? '?' : '&') + (key + "=" + filter[key]));
                }
            }
        }
        return currentUrl;
    };
    /**
     * @param {?} key
     * @param {?} data
     * @param {?} optionsList
     * @param {?} action
     * @param {?=} useDefault
     * @return {?}
     */
    RestProviderActionHandlers.prototype.getRequestUrl = /**
     * @param {?} key
     * @param {?} data
     * @param {?} optionsList
     * @param {?} action
     * @param {?=} useDefault
     * @return {?}
     */
    function (key, data, optionsList, action, useDefault) {
        /** @type {?} */
        var requestUrl;
        optionsList.forEach(function (eachOptions) {
            if (eachOptions !== undefined &&
                eachOptions.actionOptions !== undefined &&
                eachOptions.actionOptions.requestUrl !== undefined) {
                requestUrl = eachOptions.actionOptions.requestUrl(key, data, action);
                return requestUrl;
            }
        });
        if (requestUrl !== undefined) {
            return requestUrl;
        }
        /** @type {?} */
        var apiUrl = '';
        /** @type {?} */
        var pluralName = '';
        if (useDefault !== true) {
            optionsList.forEach(function (eachOptions) {
                if (eachOptions !== undefined && eachOptions.apiUrl !== undefined) {
                    apiUrl = eachOptions.apiUrl;
                }
                if (eachOptions !== undefined && eachOptions.pluralName !== undefined) {
                    pluralName = eachOptions.pluralName;
                }
            });
        }
        if (action === ProviderActionEnum.Action) {
            return apiUrl + '/' + pluralName + '/' + key;
        }
        if (action === ProviderActionEnum.Create) {
            return apiUrl + '/' + pluralName;
        }
        if (action === ProviderActionEnum.Update) {
            return apiUrl + '/' + pluralName + '/' + key;
        }
        if (action === ProviderActionEnum.Patch) {
            return apiUrl + '/' + pluralName + '/' + key;
        }
        if (action === ProviderActionEnum.Load) {
            return apiUrl + '/' + pluralName + '/' + key;
        }
        if (action === ProviderActionEnum.Delete) {
            return apiUrl + '/' + pluralName + '/' + key;
        }
        if (action === ProviderActionEnum.LoadAll) {
            return apiUrl + '/' + pluralName;
        }
    };
    /**
     * @param {?} key
     * @param {?} data
     * @param {?} optionsList
     * @param {?} action
     * @param {?=} useDefault
     * @return {?}
     */
    RestProviderActionHandlers.prototype.getRequestOptions = /**
     * @param {?} key
     * @param {?} data
     * @param {?} optionsList
     * @param {?} action
     * @param {?=} useDefault
     * @return {?}
     */
    function (key, data, optionsList, action, useDefault) {
        /** @type {?} */
        var requestOptions = {
            observe: 'response'
        };
        if (useDefault !== true) {
            optionsList.reverse().forEach(function (eachOptions) {
                if (eachOptions !== undefined &&
                    eachOptions.actionOptions !== undefined &&
                    eachOptions.actionOptions.requestOptions !== undefined) {
                    /** @type {?} */
                    var actionOptionsRequestOptions = eachOptions.actionOptions.requestOptions(key, data, action);
                    if (actionOptionsRequestOptions.headers !== undefined) {
                        requestOptions.headers = actionOptionsRequestOptions.headers;
                    }
                    if (actionOptionsRequestOptions.observe !== undefined) {
                        requestOptions.observe = actionOptionsRequestOptions.observe;
                    }
                    if (actionOptionsRequestOptions.params !== undefined) {
                        requestOptions.params = actionOptionsRequestOptions.params;
                    }
                    if (actionOptionsRequestOptions.reportProgress !== undefined) {
                        requestOptions.reportProgress = actionOptionsRequestOptions.reportProgress;
                    }
                    if (actionOptionsRequestOptions.responseType !== undefined) {
                        requestOptions.responseType = actionOptionsRequestOptions.responseType;
                    }
                    if (actionOptionsRequestOptions.withCredentials !== undefined) {
                        requestOptions.withCredentials = actionOptionsRequestOptions.withCredentials;
                    }
                }
            });
        }
        return (/** @type {?} */ (requestOptions));
    };
    /**
     * @param {?} optionsList
     * @param {?} action
     * @param {?=} useDefault
     * @return {?}
     */
    RestProviderActionHandlers.prototype.getRequestCreateType = /**
     * @param {?} optionsList
     * @param {?} action
     * @param {?=} useDefault
     * @return {?}
     */
    function (optionsList, action, useDefault) {
        /** @type {?} */
        var requestCreateType;
        if (useDefault !== true) {
            optionsList.forEach(function (eachOptions) {
                if (eachOptions !== undefined &&
                    eachOptions.actionOptions !== undefined &&
                    eachOptions.actionOptions.requestCreateType !== undefined) {
                    requestCreateType = eachOptions.actionOptions.requestCreateType(action);
                    return requestCreateType;
                }
            });
        }
        if (requestCreateType !== undefined) {
            return requestCreateType;
        }
        return 'create';
    };
    RestProviderActionHandlers.decorators = [
        { type: Injectable }
    ];
    return RestProviderActionHandlers;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var MockProviderActionHandlers = /** @class */ (function (_super) {
    __extends(MockProviderActionHandlers, _super);
    function MockProviderActionHandlers() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    MockProviderActionHandlers.decorators = [
        { type: Injectable }
    ];
    return MockProviderActionHandlers;
}(RestProviderActionHandlers));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var Model = /** @class */ (function () {
    function Model() {
    }
    Model.strings = {};
    Model.fields = [];
    Model.nested = {};
    return Model;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ProviderError = /** @class */ (function (_super) {
    __extends(ProviderError, _super);
    function ProviderError(error) {
        var _this = _super.call(this, error instanceof Error ? error.message : ((/** @type {?} */ (error)))) || this;
        _this.error = error;
        // Set the prototype explicitly.
        Object.setPrototypeOf(_this, ProviderError.prototype);
        return _this;
    }
    return ProviderError;
}(Error));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ValidatorError = /** @class */ (function (_super) {
    __extends(ValidatorError, _super);
    function ValidatorError(errors) {
        var _this = _super.call(this, Array.isArray(errors) ? JSON.stringify(errors) : ((/** @type {?} */ (errors)))) || this;
        _this.errors = errors;
        // Set the prototype explicitly.
        Object.setPrototypeOf(_this, ValidatorError.prototype);
        return _this;
    }
    return ValidatorError;
}(Error));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var parse = parseImported;
var FakeHttpClient = /** @class */ (function () {
    function FakeHttpClient(items, options) {
        this._idField = 'id';
        this._apiUrlPrefix = '/api/';
        this._pageQueryParam = 'page';
        this._limitQueryParam = 'limit';
        this._searchTextQueryParam = 'search';
        if (options) {
            if (options.idField !== undefined) {
                this._idField = options.idField;
            }
            if (options.apiUrlPrefix !== undefined) {
                this._apiUrlPrefix = options.apiUrlPrefix;
            }
            if (options.pageQueryParam !== undefined) {
                this._pageQueryParam = options.pageQueryParam;
            }
            if (options.limitQueryParam !== undefined) {
                this._limitQueryParam = options.limitQueryParam;
            }
            if (options.searchTextQueryParam !== undefined) {
                this._searchTextQueryParam = options.searchTextQueryParam;
            }
        }
        if (items !== undefined) {
            this.setItems(items);
        }
        else {
            this.mockedItems = List([]);
        }
    }
    /**
     * @param {?} items
     * @return {?}
     */
    FakeHttpClient.prototype.setItems = /**
     * @param {?} items
     * @return {?}
     */
    function (items) {
        this.mockedItems = List(items);
    };
    /**
     * @return {?}
     */
    FakeHttpClient.prototype.genId = /**
     * @return {?}
     */
    function () {
        var _this = this;
        /** @type {?} */
        var currentMax = 0;
        this.mockedItems.map(function (mapItem) {
            return Math.abs(mapItem[_this._idField]) > currentMax ? (currentMax = Math.abs(mapItem[_this._idField])) : null;
        });
        return (currentMax + 1) * -1;
    };
    /**
     * @param {?} url
     * @param {?=} data
     * @return {?}
     */
    FakeHttpClient.prototype.parseUrl = /**
     * @param {?} url
     * @param {?=} data
     * @return {?}
     */
    function (url, data) {
        /** @type {?} */
        var uri = parse(url, {}, true);
        /** @type {?} */
        var arrWithoutApi = uri.pathname.split(this._apiUrlPrefix);
        /** @type {?} */
        var arr = arrWithoutApi[arrWithoutApi.length - 1].split('/');
        /** @type {?} */
        var entityName = arr.length === 2 ? arr[arr.length - 2] : arr[arr.length - 1];
        /** @type {?} */
        var entityKey = arr.length === 2 ? arr[arr.length - 1] : undefined;
        /** @type {?} */
        var page = uri.query[this._pageQueryParam];
        /** @type {?} */
        var limit = uri.query[this._limitQueryParam];
        /** @type {?} */
        var searchText = uri.query[this._searchTextQueryParam];
        return {
            name: entityName,
            key: entityKey,
            query: uri.query,
            uri: uri,
            page: page,
            limit: limit,
            searchText: searchText
        };
    };
    /**
     * @param {?} data
     * @param {?} word
     * @return {?}
     */
    FakeHttpClient.prototype.checkWordInData = /**
     * @param {?} data
     * @param {?} word
     * @return {?}
     */
    function (data, word) {
        var _this = this;
        /** @type {?} */
        var result = Object.keys(data).filter(function (key) {
            if (data[key] instanceof Object) {
                return _this.checkWordInData(data[key], word);
            }
            if (!isNaN(+data[key])) {
                return +data[key] === +word;
            }
            return data[key] ? ((/** @type {?} */ (data[key]))).toLowerCase().indexOf(word.toLowerCase()) !== -1 : false;
        });
        return result.length > 0;
    };
    /**
     * @template T
     * @param {?} url
     * @param {?=} options
     * @return {?}
     */
    FakeHttpClient.prototype.get = /**
     * @template T
     * @param {?} url
     * @param {?=} options
     * @return {?}
     */
    function (url, options) {
        var _this = this;
        return new Observable(function (observer) {
            /** @type {?} */
            var urlData = _this.parseUrl(url);
            /** @type {?} */
            var data = {};
            /** @type {?} */
            var newBody;
            if (urlData.key) {
                /** @type {?} */
                var foundedIndex = _this.mockedItems.findIndex(function (item) {
                    return +item[_this._idField] === +urlData.key || ((/** @type {?} */ (item[_this._idField]))) === ((/** @type {?} */ (urlData.key)));
                });
                if (foundedIndex !== -1) {
                    newBody = _this.mockedItems.get(foundedIndex);
                }
            }
            else {
                /** @type {?} */
                var count_1 = 0;
                /** @type {?} */
                var xTotalCount_1 = 0;
                /** @type {?} */
                var startIndex_1 = (+urlData.page > 1 ? +urlData.page - 1 : 0) * +urlData.limit + 1;
                /** @type {?} */
                var filtredItems = _this.mockedItems.filter(function (item, index) {
                    if (urlData.searchText && !_this.checkWordInData(item, urlData.searchText)) {
                        return false;
                    }
                    else {
                        xTotalCount_1++;
                    }
                    if (index >= startIndex_1 - 1 && count_1 < +urlData.limit) {
                        count_1++;
                        return true;
                    }
                    else {
                        return false;
                    }
                });
                newBody = filtredItems.toArray();
                data.headers = new HttpHeaders({
                    'X-Total-Count': xTotalCount_1.toString()
                });
            }
            data.body = newBody;
            observer.next(data);
        });
    };
    /**
     * @template T
     * @param {?} url
     * @param {?} body
     * @param {?=} options
     * @return {?}
     */
    FakeHttpClient.prototype.post = /**
     * @template T
     * @param {?} url
     * @param {?} body
     * @param {?=} options
     * @return {?}
     */
    function (url, body, options) {
        var _this = this;
        return new Observable(function (observer) {
            if (!body[_this._idField]) {
                body[_this._idField] = _this.genId();
            }
            _this.mockedItems = _this.mockedItems.unshift(body);
            /** @type {?} */
            var data = { headers: new HttpHeaders(), body: body };
            observer.next(data);
        });
    };
    /**
     * @template T
     * @param {?} url
     * @param {?} body
     * @param {?=} options
     * @return {?}
     */
    FakeHttpClient.prototype.put = /**
     * @template T
     * @param {?} url
     * @param {?} body
     * @param {?=} options
     * @return {?}
     */
    function (url, body, options) {
        var _this = this;
        return new Observable(function (observer) {
            /** @type {?} */
            var urlData = _this.parseUrl(url);
            /** @type {?} */
            var foundedIndex = _this.mockedItems.findIndex(function (item) {
                return +item[_this._idField] === +urlData.key || ((/** @type {?} */ (item[_this._idField]))) === ((/** @type {?} */ (urlData.key)));
            });
            if (foundedIndex !== -1) {
                _this.mockedItems = _this.mockedItems.set(foundedIndex, body);
            }
            /** @type {?} */
            var data = { headers: new HttpHeaders(), body: body };
            observer.next(data);
        });
    };
    /**
     * @template T
     * @param {?} url
     * @param {?=} options
     * @return {?}
     */
    FakeHttpClient.prototype.delete = /**
     * @template T
     * @param {?} url
     * @param {?=} options
     * @return {?}
     */
    function (url, options) {
        var _this = this;
        return new Observable(function (observer) {
            /** @type {?} */
            var urlData = _this.parseUrl(url);
            /** @type {?} */
            var foundedIndex = _this.mockedItems.findIndex(function (item) {
                return +item[_this._idField] === +urlData.key || ((/** @type {?} */ (item[_this._idField]))) === ((/** @type {?} */ (urlData.key)));
            });
            if (foundedIndex !== -1) {
                _this.mockedItems = _this.mockedItems.delete(foundedIndex);
            }
            /** @type {?} */
            var data = { headers: new HttpHeaders(), body: true };
            observer.next(data);
        });
    };
    /**
     * @template T
     * @param {?} url
     * @param {?} body
     * @param {?=} options
     * @return {?}
     */
    FakeHttpClient.prototype.patch = /**
     * @template T
     * @param {?} url
     * @param {?} body
     * @param {?=} options
     * @return {?}
     */
    function (url, body, options) {
        var _this = this;
        return new Observable(function (observer) {
            /** @type {?} */
            var urlData = _this.parseUrl(url);
            /** @type {?} */
            var foundedIndex = _this.mockedItems.findIndex(function (item) {
                return +item[_this._idField] === +urlData.key || ((/** @type {?} */ (item[_this._idField]))) === ((/** @type {?} */ (urlData.key)));
            });
            if (foundedIndex !== -1) {
                /** @type {?} */
                var newBody = _this.mockedItems.get(foundedIndex);
                for (var objectKey in body) {
                    if (body[urlData.key] !== undefined) {
                        newBody[urlData.key] = body[urlData.key];
                    }
                }
                _this.mockedItems = _this.mockedItems.set(foundedIndex, newBody);
            }
            /** @type {?} */
            var data = { headers: new HttpHeaders(), body: body };
            observer.next(data);
        });
    };
    return FakeHttpClient;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template TModel
 */
var  /**
 * @template TModel
 */
Provider = /** @class */ (function () {
    function Provider(injector, factoryModel) {
        this.injector = injector;
        this.factoryModel = factoryModel;
        this.delay = 0;
        this.items$ = new BehaviorSubject(List([]));
        this.action$ = new Subject();
        this.create$ = new Subject();
        this.append$ = new Subject();
        this.update$ = new Subject();
        this.patch$ = new Subject();
        this.delete$ = new Subject();
        this.load$ = new Subject();
        this.loadAll$ = new Subject();
        this.actionIsActive$ = new BehaviorSubject(false);
        this.createIsActive$ = new BehaviorSubject(false);
        this.updateIsActive$ = new BehaviorSubject(false);
        this.deleteIsActive$ = new BehaviorSubject(false);
        this.loadIsActive$ = new BehaviorSubject(false);
        this.loadAllIsActive$ = new BehaviorSubject(false);
        this.paginationMeta$ = new BehaviorSubject(new PaginationMeta());
        this.destroy$ = new Subject();
    }
    /**
     * @param {?} data
     * @return {?}
     */
    Provider.prototype.instanceofFactoryModel = /**
     * @param {?} data
     * @return {?}
     */
    function (data) {
        return data instanceof this.factoryModel;
    };
    /**
     * @param {?} data
     * @return {?}
     */
    Provider.prototype.instanceofNestedFactoryModel = /**
     * @param {?} data
     * @return {?}
     */
    function (data) {
        var _this = this;
        return (this.factoryModel &&
            this.factoryModel.nested &&
            Object.keys(this.factoryModel.nested).filter(function (key) { return data instanceof _this.factoryModel.nested[key]; }).length > 0);
    };
    /**
     * @param {?} data
     * @return {?}
     */
    Provider.prototype.updateNestedFactoryModel = /**
     * @param {?} data
     * @return {?}
     */
    function (data) {
        var _this = this;
        Object.keys(this.factoryModel.nested).forEach(function (key) {
            return _this.items$.next(_this.items$
                .getValue()
                .map(function (item) {
                if (item[key]) {
                    if (Array.isArray(item[key])) {
                        item[key].forEach(function (eachItem, index) {
                            if (eachItem.id === data.id) {
                                item[key][index] = data;
                            }
                        });
                    }
                    else {
                        item[key] = data;
                    }
                    _this.update(item.id, item, { globalEventIsActive: false });
                }
                return item;
            })
                .toList());
        });
    };
    /**
     * @param {?} data
     * @return {?}
     */
    Provider.prototype.deleteNestedFactoryModel = /**
     * @param {?} data
     * @return {?}
     */
    function (data) {
        var _this = this;
        Object.keys(this.factoryModel.nested).forEach(function (key) {
            return _this.items$.next(_this.items$
                .getValue()
                .map(function (item) {
                if (item[key]) {
                    if (Array.isArray(item[key])) {
                        item[key] = item[key].filter(function (eachItem) { return eachItem.id !== data.id; });
                    }
                    else {
                        item[key] = undefined;
                    }
                    _this.update(item.id, item, { globalEventIsActive: false });
                }
                return item;
            })
                .toList());
        });
    };
    // tslint
    // tslint
    /**
     * @param {?} data
     * @param {?} action
     * @param {?=} classTransformOptions
     * @return {?}
     */
    Provider.prototype.plainToClass = 
    // tslint
    /**
     * @param {?} data
     * @param {?} action
     * @param {?=} classTransformOptions
     * @return {?}
     */
    function (data, action, classTransformOptions) {
        /** @type {?} */
        var model;
        if (!(data instanceof this.factoryModel)) {
            model = (/** @type {?} */ (plainToClassFromExist(new this.factoryModel(), data, classTransformOptions)));
        }
        else {
            model = (/** @type {?} */ (data));
        }
        return model;
    };
    /**
     * @param {?} model
     * @param {?} action
     * @param {?=} classTransformOptions
     * @return {?}
     */
    Provider.prototype.classToPlain = /**
     * @param {?} model
     * @param {?} action
     * @param {?=} classTransformOptions
     * @return {?}
     */
    function (model, action, classTransformOptions) {
        return classToPlainFromExist(model, {}, classTransformOptions);
    };
    /**
     * @param {?} model
     * @param {?=} classTransformOptions
     * @return {?}
     */
    Provider.prototype.classToClass = /**
     * @param {?} model
     * @param {?=} classTransformOptions
     * @return {?}
     */
    function (model, classTransformOptions) {
        return classToClass(model, classTransformOptions);
    };
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} data
     * @param {?=} options
     * @return {?}
     */
    Provider.prototype.action = /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} data
     * @param {?=} options
     * @return {?}
     */
    function (key, data, options) {
        return of(data).pipe(delay(this.delay));
    };
    /**
     * @template TProviderActionOptions
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    Provider.prototype.save = /**
     * @template TProviderActionOptions
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    function (model, options) {
        if (model.id === undefined) {
            return this.create(model, options);
        }
        return this.update(model.id, model, options);
    };
    /**
     * @template TProviderActionOptions
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    Provider.prototype.create = /**
     * @template TProviderActionOptions
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    function (model, options) {
        return of(model).pipe(delay(this.delay));
    };
    /**
     * @template TProviderActionOptions
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    Provider.prototype.append = /**
     * @template TProviderActionOptions
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    function (model, options) {
        return of(model).pipe(delay(this.delay));
    };
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    Provider.prototype.update = /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    function (key, model, options) {
        return of(model).pipe(delay(this.delay));
    };
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    Provider.prototype.patch = /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    function (key, model, options) {
        return of(model).pipe(delay(this.delay));
    };
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} options
     * @return {?}
     */
    Provider.prototype.delete = /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} options
     * @return {?}
     */
    function (key, options) {
        /** @type {?} */
        var newModel = this.plainToClass({ id: key }, ProviderActionEnum.Delete, options && options.classTransformOptions ? options.classTransformOptions : undefined);
        return of(newModel).pipe(delay(this.delay));
    };
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} options
     * @return {?}
     */
    Provider.prototype.load = /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} options
     * @return {?}
     */
    function (key, options) {
        /** @type {?} */
        var newModel = this.plainToClass({ id: key }, ProviderActionEnum.Delete, options && options.classTransformOptions ? options.classTransformOptions : undefined);
        return of(newModel).pipe(delay(this.delay));
    };
    /**
     * @template TProviderActionOptions
     * @param {?=} filter
     * @param {?=} options
     * @return {?}
     */
    Provider.prototype.loadAll = /**
     * @template TProviderActionOptions
     * @param {?=} filter
     * @param {?=} options
     * @return {?}
     */
    function (filter, options) {
        /** @type {?} */
        var newModels = [
            this.plainToClass({}, ProviderActionEnum.Delete, options && options.classTransformOptions ? options.classTransformOptions : undefined)
        ];
        return of(newModels).pipe(delay(this.delay));
    };
    /**
     * @param {?} options
     * @return {?}
     */
    Provider.prototype.calcPaginationMetaByOptions = /**
     * @param {?} options
     * @return {?}
     */
    function (options) {
        /** @type {?} */
        var paginationMeta = this.paginationMeta$.getValue();
        paginationMeta.totalResults =
            options === undefined || options.paginationMeta === undefined || options.paginationMeta.totalResults === undefined
                ? paginationMeta.totalResults
                : options.paginationMeta.totalResults;
        paginationMeta.perPage =
            options === undefined || options.paginationMeta === undefined || options.paginationMeta.perPage === undefined
                ? paginationMeta.perPage
                : options.paginationMeta.perPage;
        paginationMeta.curPage =
            options === undefined || options.paginationMeta === undefined || options.paginationMeta.curPage === undefined
                ? paginationMeta.curPage
                : options.paginationMeta.curPage;
        paginationMeta.perPage = paginationMeta.perPage === undefined ? 10 : paginationMeta.perPage;
        paginationMeta.curPage = paginationMeta.curPage === undefined ? 1 : paginationMeta.curPage;
        return this.calcPaginationMeta(paginationMeta);
    };
    /**
     * @param {?} newPaginationMeta
     * @return {?}
     */
    Provider.prototype.calcPaginationMeta = /**
     * @param {?} newPaginationMeta
     * @return {?}
     */
    function (newPaginationMeta) {
        /** @type {?} */
        var paginationMeta = this.paginationMeta$.getValue();
        if (newPaginationMeta.perPage !== undefined) {
            paginationMeta.perPage = newPaginationMeta.perPage;
        }
        if (newPaginationMeta.totalResults !== undefined) {
            paginationMeta.totalResults = newPaginationMeta.totalResults;
            paginationMeta.totalPages = Math.ceil(newPaginationMeta.totalResults / paginationMeta.perPage);
        }
        else {
            if (newPaginationMeta.totalPages !== undefined) {
                paginationMeta.totalResults = undefined;
                paginationMeta.totalPages = newPaginationMeta.totalPages;
            }
        }
        if (newPaginationMeta.curPage !== undefined) {
            paginationMeta.curPage = newPaginationMeta.curPage;
        }
        if (paginationMeta.curPage === 0) {
            paginationMeta.curPage = 1;
        }
        this.paginationMeta$.next(paginationMeta);
        return this.paginationMeta$.getValue();
    };
    /**
     * @return {?}
     */
    Provider.prototype.reloadAll = /**
     * @return {?}
     */
    function () { };
    /**
     * @return {?}
     */
    Provider.prototype.checkFilterAndOptions = /**
     * @return {?}
     */
    function () { };
    /**
     * @param {?} options
     * @return {?}
     */
    Provider.prototype.setOptions = /**
     * @param {?} options
     * @return {?}
     */
    function (options) { };
    /**
     * @return {?}
     */
    Provider.prototype.getOptions = /**
     * @return {?}
     */
    function () {
        return {};
    };
    /**
     * @param {?=} items
     * @return {?}
     */
    Provider.prototype.reconfigItems = /**
     * @param {?=} items
     * @return {?}
     */
    function (items) { };
    return Provider;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template TModel
 */
var  /**
 * @template TModel
 */
RestProvider = /** @class */ (function (_super) {
    __extends(RestProvider, _super);
    function RestProvider(injector, factoryModel, options) {
        var _this = _super.call(this, injector, factoryModel) || this;
        _this.injector = injector;
        _this.factoryModel = factoryModel;
        _this.options = options;
        _this.initFakeHttpClient();
        _this.init();
        return _this;
    }
    /**
     * @return {?}
     */
    RestProvider.prototype.initFakeHttpClient = /**
     * @return {?}
     */
    function () {
        /** @type {?} */
        var restOptions = this.options === undefined ? undefined : ((/** @type {?} */ (this.options))).restOptions;
        if (restOptions) {
            this.restOptions = restOptions;
        }
        if (!this.restOptions) {
            this.restOptions = {};
            this.restOptions.idField = 'id';
            this.restOptions.pageQueryParam = 'page';
            this.restOptions.limitQueryParam = 'limit';
            this.restOptions.searchTextQueryParam = 'search';
        }
        this.fakeHttpClient = new FakeHttpClient([], this.restOptions);
    };
    /**
     * @return {?}
     */
    RestProvider.prototype.init = /**
     * @return {?}
     */
    function () {
        this.httpClient = this.injector.get(HttpClient);
        this.providerActionHandlers = this.injector.get(RestProviderActionHandlers);
        this.setOptions((/** @type {?} */ (this.options)));
    };
    /**
     * @return {?}
     */
    RestProvider.prototype.getOptions = /**
     * @return {?}
     */
    function () {
        return this.options;
    };
    /**
     * @param {?=} options
     * @return {?}
     */
    RestProvider.prototype.setOptions = /**
     * @param {?=} options
     * @return {?}
     */
    function (options) {
        /** @type {?} */
        var filter = options === undefined ? undefined : options.filter;
        options = __assign({}, this.options, options);
        /** @type {?} */
        var autoload = options === undefined ? undefined : options.autoload;
        /** @type {?} */
        var pluralName = options === undefined ? undefined : options.pluralName;
        /** @type {?} */
        var name = options === undefined ? undefined : options.name;
        /** @type {?} */
        var apiUrl = options === undefined ? undefined : options.apiUrl;
        /** @type {?} */
        var restOptions = options === undefined ? undefined : options.restOptions;
        if (autoload !== undefined) {
            this.autoload = autoload;
        }
        if (pluralName !== undefined) {
            this.pluralName = pluralName;
        }
        if (name !== undefined) {
            this.name = name;
        }
        if (apiUrl !== undefined) {
            this.apiUrl = apiUrl;
        }
        if (filter !== undefined) {
            this.filter = __assign({}, filter);
        }
        options.autoload = this.autoload;
        options.pluralName = this.pluralName;
        options.name = this.name;
        options.apiUrl = this.apiUrl;
        options.filter = __assign({}, this.filter);
        this.options = options;
        this.calcPaginationMetaByOptions(options);
        if (this.autoload !== false) {
            this._loadAll(this.filter, this.prevOptions)
                .pipe(first())
                .subscribe();
        }
    };
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} data
     * @param {?=} options
     * @return {?}
     */
    RestProvider.prototype.action = /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} data
     * @param {?=} options
     * @return {?}
     */
    function (key, data, options) {
        var _this = this;
        this.actionIsActive$.next(true);
        /** @type {?} */
        var useDefault = false;
        // (options && options.useFakeHttpClient === true) || this.httpClient instanceof FakeHttpClient;
        /** @type {?} */
        var errors = validateSync(data, options && options.classValidatorOptions ? options.classValidatorOptions : { validationError: { target: false } });
        if (errors.length > 0 && (!options || options.globalEventIsActive !== false)) {
            return throwError(new ValidatorError(errors));
        }
        /** @type {?} */
        var optionsList = [{ actionOptions: options }, (/** @type {?} */ (this.options))];
        /** @type {?} */
        var requestUrl = this.providerActionHandlers.getRequestUrl(key, data, optionsList, ProviderActionEnum.Action, useDefault);
        /** @type {?} */
        var requestOptions = this.providerActionHandlers.getRequestOptions(key, data, optionsList, ProviderActionEnum.Action, useDefault);
        /** @type {?} */
        var request = this.providerActionHandlers.getRequest(requestUrl, data, requestOptions, optionsList, ProviderActionEnum.Action, useDefault);
        if (!request) {
            request = this.httpClient.post(requestUrl, data, requestOptions);
        }
        return request.pipe(map(function (responseData) {
            return _this.providerActionHandlers.getResponseData(responseData, optionsList, ProviderActionEnum.Action, useDefault);
        }), map(function (actionData) {
            if (options === undefined ||
                options.globalEventIsActive !== false ||
                options.globalEventIsActiveResolver !== undefined) {
                if (options === undefined ||
                    options.globalEventIsActiveResolver === undefined ||
                    options.globalEventIsActiveResolver({
                        actionKey: key,
                        requestData: data,
                        responseData: actionData
                    }, ProviderActionEnum.Action)) {
                    _this.action$.next({
                        actionKey: key,
                        requestData: data,
                        responseData: actionData
                    });
                }
            }
            _this.actionIsActive$.next(false);
            return actionData;
        }));
    };
    /**
     * @template TProviderActionOptions
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    RestProvider.prototype.create = /**
     * @template TProviderActionOptions
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    function (model, options) {
        var _this = this;
        this.createIsActive$.next(true);
        /** @type {?} */
        var useDefault = (options && options.useFakeHttpClient === true) || this.httpClient instanceof FakeHttpClient;
        model = this.plainToClass(model, ProviderActionEnum.Create, options && options.classTransformOptions ? options.classTransformOptions : undefined);
        /** @type {?} */
        var errors = validateSync(model, options && options.classValidatorOptions ? options.classValidatorOptions : { validationError: { target: false } });
        if (errors.length > 0 && (!options || options.globalEventIsActive !== false)) {
            return throwError(new ValidatorError(errors));
        }
        /** @type {?} */
        var optionsList = [{ actionOptions: options }, (/** @type {?} */ (this.options))];
        /** @type {?} */
        var isCreate = this.providerActionHandlers.getRequestCreateType(optionsList, ProviderActionEnum.Create, useDefault) === 'create';
        /** @type {?} */
        var object;
        if (isCreate === true) {
            object = this.classToPlain(model, ProviderActionEnum.Create, options && options.classTransformOptions ? options.classTransformOptions : undefined);
        }
        else {
            object = this.classToPlain(model, ProviderActionEnum.Append, options && options.classTransformOptions ? options.classTransformOptions : undefined);
        }
        /** @type {?} */
        var requestUrl = this.providerActionHandlers.getRequestUrl(undefined, object, optionsList, isCreate ? ProviderActionEnum.Create : ProviderActionEnum.Append, useDefault);
        /** @type {?} */
        var requestOptions = this.providerActionHandlers.getRequestOptions(undefined, object, optionsList, isCreate ? ProviderActionEnum.Create : ProviderActionEnum.Append, useDefault);
        /** @type {?} */
        var request = this.providerActionHandlers.getRequest(requestUrl, object, requestOptions, optionsList, isCreate ? ProviderActionEnum.Create : ProviderActionEnum.Append, useDefault);
        if (!request) {
            request = (options === undefined || options.useFakeHttpClient !== true
                ? this.httpClient
                : this.fakeHttpClient).post(requestUrl, object, requestOptions);
        }
        return request.pipe(map(function (responseData) {
            return _this.providerActionHandlers.getResponseData(responseData, optionsList, isCreate ? ProviderActionEnum.Create : ProviderActionEnum.Append, useDefault);
        }), map(function (createdItem) {
            /** @type {?} */
            var createdModel;
            if (isCreate === true) {
                createdModel = _this.plainToClass(createdItem, ProviderActionEnum.Create, options && options.classTransformOptions ? options.classTransformOptions : undefined);
                _this.items$.next(_this.items$.getValue().unshift(createdModel));
            }
            else {
                createdModel = _this.plainToClass(createdItem, ProviderActionEnum.Append, options && options.classTransformOptions ? options.classTransformOptions : undefined);
                _this.items$.next(_this.items$.getValue().push(createdModel));
            }
            /** @type {?} */
            var paginationMeta = _this.paginationMeta$.getValue();
            _this.calcPaginationMeta({ totalResults: paginationMeta.totalResults + 1 });
            _this.reconfigItems();
            if (options === undefined ||
                options.globalEventIsActive !== false ||
                options.globalEventIsActiveResolver !== undefined) {
                if (options === undefined ||
                    options.globalEventIsActiveResolver === undefined ||
                    options.globalEventIsActiveResolver(createdModel, isCreate ? ProviderActionEnum.Create : ProviderActionEnum.Append)) {
                    (isCreate ? _this.create$ : _this.append$).next(createdModel);
                }
            }
            _this.createIsActive$.next(false);
            return createdModel;
        }));
    };
    /**
     * @private
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?} model
     * @param {?} isUpdate
     * @param {?=} options
     * @return {?}
     */
    RestProvider.prototype._update = /**
     * @private
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?} model
     * @param {?} isUpdate
     * @param {?=} options
     * @return {?}
     */
    function (key, model, isUpdate, options) {
        var _this = this;
        this.updateIsActive$.next(true);
        /** @type {?} */
        var useDefault = (options && options.useFakeHttpClient === true) || this.httpClient instanceof FakeHttpClient;
        model = this.plainToClass(model, ProviderActionEnum.Update, options && options.classTransformOptions ? options.classTransformOptions : undefined);
        /** @type {?} */
        var errors = validateSync(model, options && options.classValidatorOptions ? options.classValidatorOptions : { validationError: { target: false } });
        if (errors.length > 0 && (!options || options.globalEventIsActive !== false)) {
            return throwError(new ValidatorError(errors));
        }
        /** @type {?} */
        var object;
        /** @type {?} */
        var optionsList = [{ actionOptions: options }, (/** @type {?} */ (this.options))];
        /** @type {?} */
        var request;
        if (isUpdate === true) {
            object = this.classToPlain(model, ProviderActionEnum.Update, options && options.classTransformOptions ? options.classTransformOptions : undefined);
            /** @type {?} */
            var requestUrl = this.providerActionHandlers.getRequestUrl(key, object, optionsList, ProviderActionEnum.Update, useDefault);
            /** @type {?} */
            var requestOptions = this.providerActionHandlers.getRequestOptions(key, object, optionsList, ProviderActionEnum.Update, useDefault);
            request = this.providerActionHandlers.getRequest(requestUrl, object, requestOptions, optionsList, ProviderActionEnum.Update, useDefault);
            if (!request) {
                request = (options === undefined || options.useFakeHttpClient !== true
                    ? this.httpClient
                    : this.fakeHttpClient).put(requestUrl, object, requestOptions);
            }
        }
        else {
            object = this.classToPlain(model, ProviderActionEnum.Patch, options && options.classTransformOptions ? options.classTransformOptions : undefined);
            /** @type {?} */
            var requestUrl = this.providerActionHandlers.getRequestUrl(key, object, optionsList, ProviderActionEnum.Patch, useDefault);
            /** @type {?} */
            var requestOptions = this.providerActionHandlers.getRequestOptions(key, object, optionsList, ProviderActionEnum.Patch, useDefault);
            request = this.providerActionHandlers.getRequest(requestUrl, object, requestOptions, optionsList, ProviderActionEnum.Patch, useDefault);
            if (!request) {
                request = this.httpClient.patch(requestUrl, object, requestOptions);
            }
        }
        return request.pipe(map(function (responseData) {
            return _this.providerActionHandlers.getResponseData(responseData, optionsList, isUpdate ? ProviderActionEnum.Update : ProviderActionEnum.Patch, useDefault);
        }), map(function (updatedItem) {
            /** @type {?} */
            var updatedModel = _this.plainToClass(updatedItem, isUpdate ? ProviderActionEnum.Update : ProviderActionEnum.Patch);
            /** @type {?} */
            var index = _this.items$.getValue().findIndex(function (eachModel) { return eachModel.id === key; });
            if (index !== -1) {
                _this.reconfigItems(_this.items$.getValue().set(index, updatedModel));
            }
            if (options === undefined ||
                options.globalEventIsActive !== false ||
                options.globalEventIsActiveResolver !== undefined) {
                if (options === undefined ||
                    options.globalEventIsActiveResolver === undefined ||
                    options.globalEventIsActiveResolver(updatedModel, isUpdate ? ProviderActionEnum.Update : ProviderActionEnum.Patch)) {
                    (isUpdate ? _this.update$ : _this.patch$).next(updatedModel);
                }
            }
            _this.updateIsActive$.next(false);
            return updatedModel;
        }));
    };
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    RestProvider.prototype.update = /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    function (key, model, options) {
        return this._update(key, model, true, options);
    };
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    RestProvider.prototype.patch = /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    function (key, model, options) {
        return this._update(key, model, false, options);
    };
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} options
     * @return {?}
     */
    RestProvider.prototype.load = /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} options
     * @return {?}
     */
    function (key, options) {
        var _this = this;
        this.loadIsActive$.next(true);
        /** @type {?} */
        var useDefault = (options && options.useFakeHttpClient === true) || this.httpClient instanceof FakeHttpClient;
        /** @type {?} */
        var optionsList = [{ actionOptions: options }, (/** @type {?} */ (this.options))];
        /** @type {?} */
        var requestUrl = this.providerActionHandlers.getRequestUrl(key, undefined, optionsList, ProviderActionEnum.Load, useDefault);
        /** @type {?} */
        var requestOptions = this.providerActionHandlers.getRequestOptions(key, undefined, optionsList, ProviderActionEnum.Load, useDefault);
        /** @type {?} */
        var request = this.providerActionHandlers.getRequest(requestUrl, undefined, requestOptions, optionsList, ProviderActionEnum.Load, useDefault);
        if (!request) {
            request = (options === undefined || options.useFakeHttpClient !== true
                ? this.httpClient
                : this.fakeHttpClient).get(requestUrl, requestOptions);
        }
        return request.pipe(map(function (responseData) {
            return _this.providerActionHandlers.getResponseData(responseData, optionsList, ProviderActionEnum.Load, useDefault);
        }), map(function (loadedItem) {
            /** @type {?} */
            var loadedModel = _this.plainToClass(loadedItem, ProviderActionEnum.Load, options && options.classTransformOptions ? options.classTransformOptions : undefined);
            /** @type {?} */
            var index = _this.items$.getValue().findIndex(function (eachModel) { return eachModel.id === key; });
            if (index !== -1) {
                _this.items$.next(_this.items$.getValue().set(index, loadedModel));
            }
            if (options === undefined ||
                options.globalEventIsActive !== false ||
                options.globalEventIsActiveResolver !== undefined) {
                if (options === undefined ||
                    options.globalEventIsActiveResolver === undefined ||
                    options.globalEventIsActiveResolver(loadedModel, ProviderActionEnum.Load)) {
                    _this.load$.next(loadedModel);
                }
            }
            _this.loadIsActive$.next(false);
            return loadedModel;
        }));
    };
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} options
     * @return {?}
     */
    RestProvider.prototype.delete = /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} options
     * @return {?}
     */
    function (key, options) {
        var _this = this;
        this.deleteIsActive$.next(true);
        /** @type {?} */
        var useDefault = (options && options.useFakeHttpClient === true) || this.httpClient instanceof FakeHttpClient;
        /** @type {?} */
        var optionsList = [{ actionOptions: options }, (/** @type {?} */ (this.options))];
        /** @type {?} */
        var requestUrl = this.providerActionHandlers.getRequestUrl(key, undefined, optionsList, ProviderActionEnum.Delete, useDefault);
        /** @type {?} */
        var requestOptions = this.providerActionHandlers.getRequestOptions(key, undefined, optionsList, ProviderActionEnum.Delete, useDefault);
        /** @type {?} */
        var request = this.providerActionHandlers.getRequest(requestUrl, undefined, requestOptions, optionsList, ProviderActionEnum.Delete, useDefault);
        if (!request) {
            request = (options === undefined || options.useFakeHttpClient !== true
                ? this.httpClient
                : this.fakeHttpClient).delete(requestUrl, requestOptions);
        }
        return request.pipe(map(function (responseData) {
            return _this.providerActionHandlers.getResponseData(responseData, optionsList, ProviderActionEnum.Delete, useDefault);
        }), map(function (deleted) {
            /** @type {?} */
            var index = _this.items$.getValue().findIndex(function (eachModel) { return eachModel.id === key; });
            /** @type {?} */
            var deletedModel = _this.items$.getValue().get(index);
            if (index !== -1) {
                _this.items$.next(_this.items$.getValue().delete(index));
                /** @type {?} */
                var paginationMeta = _this.paginationMeta$.getValue();
                /** @type {?} */
                var newPaginationMeta = _this.calcPaginationMeta({
                    totalResults: paginationMeta.totalResults === 0 ? 0 : paginationMeta.totalResults - 1
                });
                _this.reconfigItems();
            }
            if (options === undefined ||
                options.globalEventIsActive !== false ||
                options.globalEventIsActiveResolver !== undefined) {
                if (options === undefined ||
                    options.globalEventIsActiveResolver === undefined ||
                    options.globalEventIsActiveResolver(deletedModel, ProviderActionEnum.Delete)) {
                    _this.delete$.next(deletedModel);
                }
            }
            _this.deleteIsActive$.next(false);
            return deletedModel;
        }));
    };
    /**
     * @return {?}
     */
    RestProvider.prototype.reloadAll = /**
     * @return {?}
     */
    function () {
        this._loadAll(this.filter, this.prevOptions)
            .pipe(first())
            .subscribe();
    };
    /**
     * @template TProviderActionOptions
     * @param {?=} filter
     * @param {?=} options
     * @return {?}
     */
    RestProvider.prototype.loadAll = /**
     * @template TProviderActionOptions
     * @param {?=} filter
     * @param {?=} options
     * @return {?}
     */
    function (filter, options) {
        filter = __assign({}, filter);
        /** @type {?} */
        var searchText = filter[this.restOptions.searchTextQueryParam] || filter.searchText;
        delete filter.searchText;
        this.calcPaginationMeta({
            curPage: filter[this.restOptions.pageQueryParam] || filter.curPage,
            perPage: filter[this.restOptions.limitQueryParam] || filter.perPage
        });
        delete filter.curPage;
        delete filter.perPage;
        filter[this.restOptions.searchTextQueryParam] = searchText;
        return this._loadAll(filter, options);
    };
    /**
     * @private
     * @template TProviderActionOptions
     * @param {?=} filter
     * @param {?=} options
     * @return {?}
     */
    RestProvider.prototype._loadAll = /**
     * @private
     * @template TProviderActionOptions
     * @param {?=} filter
     * @param {?=} options
     * @return {?}
     */
    function (filter, options) {
        var _this = this;
        this.loadAllIsActive$.next(true);
        /** @type {?} */
        var useDefault = (options && options.useFakeHttpClient === true) || this.httpClient instanceof FakeHttpClient;
        /** @type {?} */
        var optionsList = [{ actionOptions: options }, (/** @type {?} */ (this.options))];
        /** @type {?} */
        var paginationMeta = this.paginationMeta$.getValue();
        this.filter = filter ? __assign({}, filter) : {};
        for (var key in this.filter) {
            if (this.filter.hasOwnProperty(key) && !this.filter[key]) {
                delete this.filter[key];
            }
        }
        this.prevOptions = options;
        /** @type {?} */
        var requestUrl = this.providerActionHandlers.getRequestUrl(undefined, this.filter, optionsList, ProviderActionEnum.LoadAll, useDefault);
        this.filter[this.restOptions.pageQueryParam] = paginationMeta.curPage;
        this.filter[this.restOptions.limitQueryParam] = paginationMeta.perPage;
        requestUrl = this.providerActionHandlers.getRequestQuery(requestUrl, this.filter, optionsList, ProviderActionEnum.LoadAll, useDefault);
        /** @type {?} */
        var requestOptions = this.providerActionHandlers.getRequestOptions(undefined, this.filter, optionsList, ProviderActionEnum.LoadAll, useDefault);
        /** @type {?} */
        var request = this.providerActionHandlers.getRequest(requestUrl, undefined, requestOptions, optionsList, ProviderActionEnum.LoadAll, useDefault);
        if (!request) {
            request = (options === undefined || options.useFakeHttpClient !== true
                ? this.httpClient
                : this.fakeHttpClient).get(requestUrl, requestOptions);
        }
        return request.pipe(map(function (responseData) {
            /** @type {?} */
            var responsePaginationMeta = _this.providerActionHandlers.getResponsePaginationMeta(responseData, optionsList, ProviderActionEnum.LoadAll, useDefault);
            if (responsePaginationMeta) {
                _this.calcPaginationMeta(__assign({ curPage: paginationMeta.curPage, perPage: paginationMeta.perPage }, responsePaginationMeta));
            }
            else {
                _this.calcPaginationMeta({
                    curPage: paginationMeta.curPage,
                    perPage: paginationMeta.perPage
                });
            }
            return _this.providerActionHandlers.getResponseData(responseData, optionsList, ProviderActionEnum.LoadAll, useDefault);
        }), map(function (loadedItems) {
            /** @type {?} */
            var newPaginationMeta = _this.paginationMeta$.getValue();
            /** @type {?} */
            var loadedModels = loadedItems === undefined
                ? []
                : loadedItems.map(function (loadedItem) {
                    return _this.plainToClass(loadedItem, ProviderActionEnum.LoadAll, options && options.classTransformOptions ? options.classTransformOptions : undefined);
                });
            if (_this.options.infinity && newPaginationMeta.curPage > 1) {
                /** @type {?} */
                var items = _this.items$.getValue().toArray();
                loadedModels = __spread(items, loadedModels);
            }
            _this.reconfigItems(List(loadedModels));
            if (options === undefined ||
                options.globalEventIsActive !== false ||
                options.globalEventIsActiveResolver !== undefined) {
                if (options === undefined ||
                    options.globalEventIsActiveResolver === undefined ||
                    options.globalEventIsActiveResolver(loadedModels, ProviderActionEnum.LoadAll)) {
                    _this.loadAll$.next(loadedModels);
                }
            }
            _this.loadAllIsActive$.next(false);
            return loadedModels;
        }));
    };
    /**
     * @param {?=} items
     * @return {?}
     */
    RestProvider.prototype.reconfigItems = /**
     * @param {?=} items
     * @return {?}
     */
    function (items) {
        /** @type {?} */
        var newPaginationMeta = this.paginationMeta$.getValue();
        if (newPaginationMeta.perPage === undefined) {
            throw new ProviderError('Not set perPage count');
        }
        if (this.options.infinity && newPaginationMeta.curPage > 1) {
            if (items) {
                this.items$.next(items);
            }
            else {
                this.items$.next(this.items$.getValue());
            }
        }
        else {
            if (items) {
                this.items$.next(List(items.take(newPaginationMeta.perPage)));
            }
            else {
                this.items$.next(List(this.items$.getValue().take(newPaginationMeta.perPage)));
            }
        }
    };
    return RestProvider;
}(Provider));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template TModel
 */
var  /**
 * @template TModel
 */
MockProvider = /** @class */ (function (_super) {
    __extends(MockProvider, _super);
    function MockProvider(injector, factoryModel, options) {
        var _this = _super.call(this, injector, factoryModel, options) || this;
        _this.injector = injector;
        _this.factoryModel = factoryModel;
        _this.options = options;
        return _this;
    }
    /**
     * @return {?}
     */
    MockProvider.prototype.init = /**
     * @return {?}
     */
    function () {
        this.httpClient = this.fakeHttpClient;
        this.providerActionHandlers = this.injector.get(MockProviderActionHandlers);
        this.setOptions((/** @type {?} */ (this.options)));
    };
    /**
     * @param {?=} options
     * @return {?}
     */
    MockProvider.prototype.setOptions = /**
     * @param {?=} options
     * @return {?}
     */
    function (options) {
        if (this.apiUrl === undefined) {
            this.apiUrl = 'http://fake/api';
        }
        if (this.pluralName === undefined) {
            this.pluralName = 'entities';
        }
        if (this.name === undefined) {
            this.name = 'entity';
        }
        if (options && options.items) {
            ((/** @type {?} */ (this.httpClient))).setItems(options.items);
        }
        _super.prototype.setOptions.call(this, options);
    };
    return MockProvider;
}(RestProvider));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template TModel
 */
var  /**
 * @template TModel
 */
Repository = /** @class */ (function () {
    function Repository(injector, factoryModel, factoryMockProvider, factoryRestProvider) {
        this.injector = injector;
        this.factoryModel = factoryModel;
        this.factoryMockProvider = factoryMockProvider;
        this.factoryRestProvider = factoryRestProvider;
        this.mockProviders = [];
        this.restProviders = [];
        this._mockProviderIsActive = false;
        this._restProviderIsActive = false;
        this.destroyed$ = new Subject();
    }
    Object.defineProperty(Repository.prototype, "provider", {
        get: /**
         * @return {?}
         */
        function () {
            if (this._mockProviderIsActive && this.mockProvider !== undefined) {
                return this.mockProvider;
            }
            if (this._restProviderIsActive && this.restProvider !== undefined) {
                return this.restProvider;
            }
            return new Provider(this.injector, this.factoryModel);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Repository.prototype, "paginationMeta$", {
        get: /**
         * @return {?}
         */
        function () {
            return this.provider.paginationMeta$;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Repository.prototype, "providers", {
        get: /**
         * @return {?}
         */
        function () {
            return __spread([this.restProvider], this.restProviders, [this.mockProvider], this.mockProviders);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Repository.prototype, "items", {
        get: /**
         * @return {?}
         */
        function () {
            return this.provider.items$.getValue().toArray();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Repository.prototype, "items$", {
        get: /**
         * @return {?}
         */
        function () {
            return this.provider.items$.pipe(map(function (items) { return items.toArray(); }));
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    Repository.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.destroyed$.next(true);
        this.destroyed$.complete();
    };
    /**
     * @template TNewModel
     * @param {?=} newFactoryModel
     * @param {?=} newFactoryMockProvider
     * @param {?=} newFactoryRestProvider
     * @return {?}
     */
    Repository.prototype.fork = /**
     * @template TNewModel
     * @param {?=} newFactoryModel
     * @param {?=} newFactoryMockProvider
     * @param {?=} newFactoryRestProvider
     * @return {?}
     */
    function (newFactoryModel, newFactoryMockProvider, newFactoryRestProvider) {
        if (this.root !== undefined) {
            return this.root.fork(newFactoryModel, newFactoryMockProvider, newFactoryRestProvider);
        }
        /** @type {?} */
        var newInstance = new Repository(this.injector, newFactoryModel ? newFactoryModel : ((/** @type {?} */ (this.factoryModel))), newFactoryMockProvider ? newFactoryMockProvider : ((/** @type {?} */ (this.factoryMockProvider))), newFactoryRestProvider ? newFactoryRestProvider : ((/** @type {?} */ (this.factoryRestProvider))));
        newInstance.root = (/** @type {?} */ (this));
        return newInstance;
    };
    /**
     * @template TMockProviderOptions
     * @param {?=} options
     * @return {?}
     */
    Repository.prototype.useMock = /**
     * @template TMockProviderOptions
     * @param {?=} options
     * @return {?}
     */
    function (options) {
        this._restProviderIsActive = false;
        this._mockProviderIsActive = true;
        if (this.mockProvider === undefined && options !== undefined) {
            this.mockProvider = (/** @type {?} */ ((this.factoryMockProvider
                ? new this.factoryMockProvider(this.injector, this.factoryModel, options)
                : new MockProvider(this.injector, this.factoryModel, options))));
            this.mockProvider.name = 'Mock#' + this.root.mockProviders.length.toString();
            if (this.root !== undefined) {
                this.root.subscribeToProvider(this.mockProvider);
                this.root.mockProviders.push(this.mockProvider);
            }
        }
        else {
            this.mockProvider.name = 'Mock#root';
            this.mockProvider.setOptions(options);
            this.subscribeToProvider(this.mockProvider);
        }
    };
    /**
     * @template TMockProviderOptions
     * @param {?=} options
     * @return {?}
     */
    Repository.prototype.useRest = /**
     * @template TMockProviderOptions
     * @param {?=} options
     * @return {?}
     */
    function (options) {
        // tslint:disable-line
        this._restProviderIsActive = true;
        this._mockProviderIsActive = false;
        if (this.restProvider === undefined && options !== undefined) {
            this.restProvider = (/** @type {?} */ ((this.factoryRestProvider
                ? new this.factoryRestProvider(this.injector, this.factoryModel, options)
                : new RestProvider(this.injector, this.factoryModel, options))));
            this.restProvider.name = 'Rest#' + this.root.restProviders.length.toString();
            if (this.root !== undefined) {
                this.root.subscribeToProvider(this.restProvider);
                this.root.restProviders.push(this.restProvider);
            }
        }
        else {
            this.restProvider.name = 'Rest#root';
            this.restProvider.setOptions(options);
            this.subscribeToProvider(this.restProvider);
        }
    };
    /**
     * @param {?} provider
     * @return {?}
     */
    Repository.prototype.subscribeToProvider = /**
     * @param {?} provider
     * @return {?}
     */
    function (provider) {
        var _this = this;
        provider.create$.pipe(takeUntil(this.destroyed$)).subscribe(function (item) {
            _this.providers.forEach(function (eachProvider) {
                /** @type {?} */
                var options = eachProvider ? eachProvider.getOptions() : undefined;
                if (eachProvider !== undefined &&
                    eachProvider.name !== provider.name &&
                    eachProvider.instanceofFactoryModel(item) &&
                    (options === undefined ||
                        (options.globalEventResolver === undefined || options.globalEventResolver(item, ProviderActionEnum.Create)))) {
                    eachProvider
                        .create(item, { useFakeHttpClient: true, globalEventIsActive: false })
                        .pipe(first())
                        .subscribe();
                }
            });
        });
        provider.append$.pipe(takeUntil(this.destroyed$)).subscribe(function (item) {
            _this.providers.forEach(function (eachProvider) {
                /** @type {?} */
                var options = eachProvider ? eachProvider.getOptions() : undefined;
                if (eachProvider !== undefined &&
                    eachProvider.name !== provider.name &&
                    eachProvider.instanceofFactoryModel(item) &&
                    (options === undefined ||
                        (options.globalEventResolver === undefined || options.globalEventResolver(item, ProviderActionEnum.Create)))) {
                    eachProvider
                        .append(item, { useFakeHttpClient: true, globalEventIsActive: false })
                        .pipe(first())
                        .subscribe();
                }
            });
        });
        provider.update$.pipe(takeUntil(this.destroyed$)).subscribe(function (item) {
            _this.providers.forEach(function (eachProvider) {
                /** @type {?} */
                var options = eachProvider ? eachProvider.getOptions() : undefined;
                if (eachProvider !== undefined && eachProvider.name !== provider.name) {
                    if (eachProvider.instanceofFactoryModel(item) &&
                        (options === undefined ||
                            (options.globalEventResolver === undefined ||
                                options.globalEventResolver(item, ProviderActionEnum.Update)))) {
                        eachProvider
                            .update(item.id, item, { useFakeHttpClient: true, globalEventIsActive: false })
                            .pipe(first())
                            .subscribe();
                    }
                    if (eachProvider.instanceofNestedFactoryModel(item)) {
                        eachProvider.updateNestedFactoryModel(item);
                    }
                }
            });
        });
        provider.patch$.pipe(takeUntil(this.destroyed$)).subscribe(function (item) {
            _this.providers.forEach(function (eachProvider) {
                /** @type {?} */
                var options = eachProvider ? eachProvider.getOptions() : undefined;
                if (eachProvider !== undefined && eachProvider.name !== provider.name) {
                    if (eachProvider.instanceofFactoryModel(item) &&
                        (options === undefined ||
                            (options.globalEventResolver === undefined ||
                                options.globalEventResolver(item, ProviderActionEnum.Patch)))) {
                        eachProvider
                            .patch(item.id, item, { useFakeHttpClient: true, globalEventIsActive: false })
                            .pipe(first())
                            .subscribe();
                    }
                }
            });
        });
        provider.delete$.pipe(takeUntil(this.destroyed$)).subscribe(function (item) {
            _this.providers.forEach(function (eachProvider) {
                /** @type {?} */
                var options = eachProvider ? eachProvider.getOptions() : undefined;
                if (eachProvider !== undefined && eachProvider.name !== provider.name) {
                    if (eachProvider.instanceofFactoryModel(item) &&
                        (options === undefined ||
                            (options.globalEventResolver === undefined ||
                                options.globalEventResolver(item, ProviderActionEnum.Delete)))) {
                        eachProvider
                            .delete(item.id, { useFakeHttpClient: true, globalEventIsActive: false })
                            .pipe(first())
                            .subscribe();
                    }
                    if (eachProvider.instanceofNestedFactoryModel(item)) {
                        eachProvider.deleteNestedFactoryModel(item);
                    }
                }
            });
        });
        provider.load$.pipe(takeUntil(this.destroyed$)).subscribe(function (item) {
            _this.providers.forEach(function (eachProvider) {
                /** @type {?} */
                var options = eachProvider ? eachProvider.getOptions() : undefined;
                if (eachProvider !== undefined &&
                    eachProvider.name !== provider.name &&
                    eachProvider.instanceofFactoryModel(item) &&
                    (options === undefined ||
                        (options.globalEventResolver === undefined || options.globalEventResolver(item, ProviderActionEnum.Update)))) {
                    eachProvider
                        .update(item.id, item, { useFakeHttpClient: true, globalEventIsActive: false })
                        .pipe(first())
                        .subscribe();
                }
            });
        });
        provider.loadAll$.pipe(takeUntil(this.destroyed$)).subscribe(function (items) {
            items.forEach(function (item) {
                return _this.providers.forEach(function (eachProvider) {
                    /** @type {?} */
                    var options = eachProvider ? eachProvider.getOptions() : undefined;
                    if (eachProvider !== undefined &&
                        eachProvider.name !== provider.name &&
                        eachProvider.instanceofFactoryModel(item) &&
                        (options === undefined ||
                            (options.globalEventResolver === undefined ||
                                options.globalEventResolver(item, ProviderActionEnum.Update)))) {
                        eachProvider
                            .update(item.id, item, { useFakeHttpClient: true, globalEventIsActive: false })
                            .pipe(first())
                            .subscribe();
                    }
                });
            });
        });
    };
    /**
     * @param {?} options
     * @return {?}
     */
    Repository.prototype.setOptions = /**
     * @param {?} options
     * @return {?}
     */
    function (options) {
        this.provider.setOptions(options);
    };
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} data
     * @param {?=} options
     * @return {?}
     */
    Repository.prototype.action = /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} data
     * @param {?=} options
     * @return {?}
     */
    function (key, data, options) {
        return this.provider.action(key, data, options).pipe(first());
    };
    /**
     * @template TProviderActionOptions
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    Repository.prototype.save = /**
     * @template TProviderActionOptions
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    function (model, options) {
        return this.provider.save(model, options).pipe(first());
    };
    /**
     * @template TProviderActionOptions
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    Repository.prototype.create = /**
     * @template TProviderActionOptions
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    function (model, options) {
        return this.provider.create(model, options).pipe(first());
    };
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    Repository.prototype.update = /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    function (key, model, options) {
        return this.provider.update(key, model, options).pipe(first());
    };
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    Repository.prototype.patch = /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    function (key, model, options) {
        return this.provider.patch(key, model, options).pipe(first());
    };
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} options
     * @return {?}
     */
    Repository.prototype.load = /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} options
     * @return {?}
     */
    function (key, options) {
        return this.provider.load(key, options).pipe(first());
    };
    /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} options
     * @return {?}
     */
    Repository.prototype.delete = /**
     * @template TProviderActionOptions
     * @param {?} key
     * @param {?=} options
     * @return {?}
     */
    function (key, options) {
        return this.provider.delete(key, options).pipe(first());
    };
    /**
     * @return {?}
     */
    Repository.prototype.reloadAll = /**
     * @return {?}
     */
    function () {
        this.provider.reloadAll();
    };
    /**
     * @template TProviderActionOptions
     * @param {?=} filter
     * @param {?=} options
     * @return {?}
     */
    Repository.prototype.loadAll = /**
     * @template TProviderActionOptions
     * @param {?=} filter
     * @param {?=} options
     * @return {?}
     */
    function (filter, options) {
        return this.provider.loadAll(filter, options);
    };
    /**
     * @param {?} model
     * @return {?}
     */
    Repository.prototype.clone = /**
     * @param {?} model
     * @return {?}
     */
    function (model) {
        return this.provider.classToClass(model);
    };
    return Repository;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var DynamicRepository = /** @class */ (function (_super) {
    __extends(DynamicRepository, _super);
    function DynamicRepository(injector) {
        var _this = _super.call(this, injector, Model) || this;
        _this.injector = injector;
        return _this;
    }
    DynamicRepository.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    DynamicRepository.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    return DynamicRepository;
}(Repository));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var NgxRepositoryModule = /** @class */ (function () {
    function NgxRepositoryModule() {
    }
    /**
     * @return {?}
     */
    NgxRepositoryModule.forRoot = /**
     * @return {?}
     */
    function () {
        return {
            ngModule: NgxRepositoryModule,
            providers: []
        };
    };
    NgxRepositoryModule.decorators = [
        { type: NgModule, args: [{
                    imports: [CommonModule],
                    providers: [RestProviderActionHandlers, MockProviderActionHandlers, DynamicRepository]
                },] }
    ];
    return NgxRepositoryModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NgxRepositoryModule, ProviderActionEnum, ProviderError, ValidatorError, Model, PaginationMeta, MockProviderActionHandlers, MockProvider, Provider, RestProviderActionHandlers, RestProvider, DynamicRepository, Repository, FakeHttpClient };

//# sourceMappingURL=ngx-repository.js.map